<?php

namespace PR\REST_API\SHIPANY_Asia;

use Exception;
use PR\REST_API\API_Client;
use PR\REST_API\Interfaces\API_Auth_Interface;
use PR\REST_API\Interfaces\API_Driver_Interface;
use SHIPANY\Utils\Args_Parser;
use stdClass;

/**
 * The API client for eCS.
 *
 * @since [*next-version*]
 */
class Client extends API_Client {
	/**
	 *
	 */
	protected $pickup_id;

	/**
	 *
	 */
	protected $soldto_id;

	/**
	 *
	 */
	protected $shipping_shipnay_settings;

	/**
	 * The language of the message
	 *
	 */
	protected $language = 'en';

	/**
	 * The version of the message
	 *
	 */
	protected $version = '1.4';

	/**
	 * {@inheritdoc}
	 *
	 * @since [*next-version*]
	 *
	 * @param string $contact_name The contact name to use for creating orders.
	 */
	public function __construct( $pickup_id, $soldto_id, $base_url, API_Driver_Interface $driver, API_Auth_Interface $auth = null ) {
		parent::__construct( $base_url, $driver, $auth );

		$this->pickup_id = $pickup_id;
		$this->soldto_id = $soldto_id;

	}

	/**
	 * Create shipping label
	 *
	 * @since [*next-version*]
	 *
	 * @param int $order_id The order id.
	 *
	 */
	public function create_label( Item_Info $item_info ){
		$shipping_shipnay_settings = SHIPANY()->get_shipping_shipany_settings();
		$route 	= $this->shipping_label_route();

		//$data = $this->item_info_to_request_data( $item_info );
		//shipany
		$data = $this->item_info_to_request_data_shipany( $item_info );

		$response = $this->post($route, $data);
		//$response_body = json_decode( $response->body );
		//shipany
		$response_body = $response->body;
		
		/*
		if ( $response->status == 200 ) {

			if( $this->check_status_code( $response_body ) == 200 ){

				return $this->get_label_content( $response_body );

			}
		}
		*/
		//shipany
		//no matter draft or not, api always return 201 when order created
		if ( $response->status === 201 ) {
			return $response_body->data->objects[0];
		}

		throw new Exception(
			sprintf(
				__( 'Failed to create ShipAny order: %s', 'pr-shipping-shipany'),
				$this->generate_error_details( $response_body )
			)
		);
	}

	public function send_pickup( $order_id ){
		$route = 'orders/' . $order_id . '/ ';
		$data = array(
			'ops' => 
			array(
			  0 => 
			  array(
				'op' => 'add',
				'path' => '/states/0',
				'value' => 
				array(
				  'stat' => 'Pickup Request Sent',
				),
			  ),
			),
		);
		$response = $this->patch($route, json_encode($data));
		if ( $response->status === 200 ) {
			return $response;
		}
	}

	public function close_out_labels( $country_code, $shipment_ids = array() ){

		$route 			= $this->close_out_label_route();
		$shipment_items = array();
		
		if( is_array( $shipment_ids ) && count( $shipment_ids ) > 0 ){

			foreach( $shipment_ids as $shipment_id ){
				$shipment_items[] = array(
					'shipmentID' => $shipment_id,
				);
			}

		}

		$data 		= array(
			'closeOutRequest' 	=> array(
				'hdr' 	=> array(
					'messageType' 		=> $this->get_type( 'closeout' ),
					'messageDateTime' 	=> $this->get_datetime(),
					'messageVersion' 	=> '1.3',
					'messageLanguage' 	=> $this->get_language()
				),
				'bd' 	=> array(
					'pickupAccountId' 	=> $this->pickup_id,
					'soldToAccountId'	=> $this->soldto_id,
					'generateHandover' 	=> 'Y',
					'handoverMethod' 	=> 1,
				)
			)
		);

		if( !in_array( $country_code, array('IN', 'CN', 'HK', 'AU', 'SG', 'MY', 'TH') ) ){
			$data['closeOutRequest']['bd']['handoverID'] = 'C' . date("YmdHis");
			$data['closeOutRequest']['bd']['generateHandover'] = 'N';
		}

		if( count( $shipment_items ) > 0 ){
			$data['closeOutRequest']['bd']['shipmentItems'] = $shipment_items;
		}

		$response 		= $this->post($route, $data );
		$response_body 	= json_decode( $response->body );

		if ( $response->status === 200 ) {

			$status_code = $this->check_status_code( $response_body, 'closeOutResponse' );
			
			if( $status_code == 200 || $status_code == 204 ){

				return $this->get_closeout_content( $response_body, $shipment_ids );

			}
		}

		throw new Exception(
			sprintf(
				__( 'Failed to close out label: %s', 'pr-shipping-shipany' ),
				$this->generate_error_details( $response_body, 'closeOutResponse' )
			)
		);

	}

	public function check_status_code( $label_response, $response_type = 'labelResponse' ){
		
		if( !isset( $label_response->$response_type->bd->responseStatus->code ) ){
			throw new Exception( __( 'Response status is not exist!', 'pr-shipping-shipany' ) );
		}

		return $label_response->$response_type->bd->responseStatus->code;
	}

	public function get_label_content( $label_response ){

		if( !isset( $label_response->labelResponse->bd->labels ) ){
			throw new Exception( __( 'Label info is not exist!', 'pr-shipping-shipany' ) );
		}

		$labels_info 		= $label_response->labelResponse->bd->labels;

		foreach( $labels_info as $info ){

			if( !isset( $info->content ) ){
				throw new Exception( __( 'Label content is not exist!', 'pr-shipping-shipany' ) );
			}elseif( !isset( $info->shipmentID ) ){
				throw new Exception( __( 'Shipment ID is not exist!', 'pr-shipping-shipany' ) );
			}else{

				return $info;

			}
		}

		return false;
	}

	public function get_closeout_content( $response, $shipment_ids ){

		if( !isset( $response->closeOutResponse->bd->responseStatus->messageDetails ) && count( $shipment_ids ) < 1 ){
			throw new Exception( __( 'Message Detail does not exist!', 'pr-shipping-shipany' ) );
		}

		return $response->closeOutResponse->bd;
	}

	public function generate_error_details( $label_response, $response_type = 'labelResponse' ){

		$error_details 	= '';

		if( $response_type == 'labelResponse' ){

			if( isset( $label_response->$response_type->bd->labels ) ) {

				$labels = $label_response->$response_type->bd->labels;

				$error_details .= $this->get_error_lists( $labels );
	
			}

		}elseif( $response_type == 'closeOutResponse' || $response_type == 'deleteShipmentResp' ){

			if( isset( $label_response->$response_type->bd->shipmentItems ) ) {

				$items = $label_response->$response_type->bd->shipmentItems;
	
				$error_details .= $this->get_error_lists( $items );
			}

		}
		

		$error_exception = '';
		$response_status = $label_response->$response_type->bd->responseStatus;

		if( isset( $response_status->message) ){
			$error_exception .= $response_status->message  . '<br /> ';
		}

		if( isset( $response_status->messageDetails[0]->messageDetail ) ){

			$message_details = '';
			foreach( $response_status->messageDetails as $message_detail ){

				if( isset( $message_detail->messageDetail ) ){
					$message_details .= $message_detail->messageDetail . '<br />';
				}

			}

			if( !empty( $message_detail ) ){

				$error_exception .= 'Error Details: ' . $message_details;

			}

		}

		if( !empty( $error_details ) ){

			$error_exception .= '<ul class = "wc_shipany_error">' . $error_details . '</ul>';

		}else{

			$error_exception .= __( 'Error message detail is not exis1t!', 'pr-shipping-shipany' );

		}

		return $error_exception;
	}

	public function get_error_lists( $items ){

		$error_details = '';

		foreach( $items as $item ){
	
			if( !isset( $item->responseStatus->messageDetails ) ){
				continue;
			}

			$shipment_id_text = '';
			if( isset( $item->shipmentID ) ){
				$shipment_id_text = $item->shipmentID . ' - ';
			}

			foreach( $item->responseStatus->messageDetails as $message_detail ){

				if( isset( $message_detail->messageDetail ) ){

					$error_details .= '<li>' . $shipment_id_text . $message_detail->messageDetail . '</li>';

				}

			}

		}

		return $error_details;
	}

	/**
	 * Get message type.
	 *
	 * @param string $type The type of the message.
	 *
	 * @return string The type of the message.
	 */
	protected function get_type( $type = 'create' ){

		if( $type == 'delete' ) {
			return 'DELETESHIPMENT';
		}elseif( $type == 'closeout' ){
			return 'CLOSEOUT';
		}elseif( $type == 'create' ) {
			return 'LABEL';
		}
	}

	/**
	 * Get date time.
	 *
	 * @return string The date and time of the message.
	 */
	protected function get_datetime(){
		return date( 'c', time() );
	}

	/**
	 * Get message language.
	 *
	 * @return string The language of the message.
	 */
	protected function get_language(){
		return $this->language;
	}

	/**
 * Get message version.
 *
 * @return string The version of the message.
 */
    protected function get_version(){
        return $this->version;
    }

    /**
     * Get message version.
     *
     * @return string The version of the message.
     */
    protected function get_shipment_id( $prefix, $id ){
        $prefix = trim( $prefix );
        return $prefix . $id . time();
    }


	protected function query_rate( $received_data ) {
		$route = 'couriers-connector/query-rate/ ';
		$data = array(
			"wt" => array(
				"val" => $received_data["wt"]["val"],
				"unt" => $received_data["wt"]["unt"]
			),
			"dim" => array(
				"len" => $received_data["dim"]["len"],
				"wid" => $received_data["dim"]["wid"],
				"hgt" => $received_data["dim"]["hgt"],
				"unt" => $received_data["dim"]["unt"]
			),
			"items" => array(
				array(
					"sku" => $received_data["items"][0]["sku"],
					"name" => $received_data["items"][0]["name"],
					"typ" => "",
					"descr" => "",
					"ori" => "",
					"unt_price" => array(
						"val" => $received_data["items"][0]["unt_price"]["val"],
						"ccy" => "HKD"
					),
					"qty" => $received_data["items"][0]["qty"],
					"wt" => array(
						"val" => $received_data["items"][0]["wt"]["val"],
						"unt" => $received_data["items"][0]["wt"]["unt"]
					),
					"dim" => array(
						"len" => $received_data["items"][0]["dim"]["len"],
						"wid" => $received_data["items"][0]["dim"]["wid"],
						"hgt" => $received_data["items"][0]["dim"]["hgt"],
						"unt" => $received_data["items"][0]["dim"]["unt"]
					),
					"stg" => "Normal"
				)
			),
			"mch_ttl_val" => array(
				"val" => $received_data["mch_ttl_val"]["val"],
				"ccy" => "HKD"
			),
			"sndr_ctc" => array(
				"ctc" => array(
					"tit" => "",
					"f_name" => $received_data["sndr_ctc"]["ctc"]["f_name"],
					"l_name" => $received_data["sndr_ctc"]["ctc"]["l_name"],
					"phs" => array(
						array(
							"typ" => $received_data["sndr_ctc"]["ctc"]["phs"][0]["typ"],
							"cnty_code" => $received_data["sndr_ctc"]["ctc"]["phs"][0]["cnty_code"],
							"ar_code" => "",
							"num" => $received_data["sndr_ctc"]["ctc"]["phs"][0]["num"],
							"ext_no" => ""
						)
					),
					"email" => "",
					"note" => ""
				),
				"addr" => array(
					"typ" => $received_data["sndr_ctc"]["addr"]["typ"],
					"ln" => $received_data["sndr_ctc"]["addr"]["ln"],
					"ln2" => $received_data["sndr_ctc"]["addr"]["ln2"],
					"ln3" => "",
					"distr" => $received_data["sndr_ctc"]["addr"]["distr"],
					"city" => $received_data["sndr_ctc"]["addr"]["cnty"],
					"state" => "",
					"cnty" => "HKG",
					"zc" => ""
				)
			),
			"rcvr_ctc" => array(
				"ctc" => array(
					"tit" => "",
					"f_name" => $received_data["rcvr_ctc"]["ctc"]["f_name"],
					"l_name" => $received_data["rcvr_ctc"]["ctc"]["l_name"],
					"phs" => array(
						array(
							"typ" => $received_data["rcvr_ctc"]["ctc"]["phs"][0]["typ"],
							"cnty_code" => $received_data["rcvr_ctc"]["ctc"]["phs"][0]["cnty_code"],
							"ar_code" => "",
							"num" => $received_data["rcvr_ctc"]["ctc"]["phs"][0]["num"],
							"ext_no" => ""
						)
					),
					"email" => $received_data["rcvr_ctc"]["ctc"]["email"],
					"note" => ""
				),
				"addr" => array(
					"typ" => $received_data["rcvr_ctc"]["addr"]["typ"],
					"ln" => $received_data["rcvr_ctc"]["addr"]["ln"],
					"ln2" => $received_data["sndr_ctc"]["addr"]["ln2"],
					"ln3" => "",
					"distr" => "",
					"city" => $received_data["rcvr_ctc"]["addr"]["cnty"],
					"state" => "",
					"cnty" => "HKG",
					"zc" => ""
				)
			)
		);

		$response = $this->post($route, $data);
		if ( $response->status === 200 ) {
			return $response->body->data->objects[0]->quots;//$response->body->data->objects[0]->quots[0]->quot_uid
		}
	}

	/**
	 * Transforms an item info object into a request data array.
	 *
	 * @param Item_Info $item_info The item info object to transform.
	 *
	 * @return array The request data for the given item info object.
	 */
	protected function item_info_to_request_data( Item_Info $item_info ) {
		$shipmentid 		= $this->get_shipment_id( $item_info->shipment['prefix'], $item_info->shipment['order_id'] );

		$shipment_item 		= array(
			//'returnAddress' 	=> $return_address,
//			'consigneeAddress' 	=> $this->get_consignee( $item_info ),
			'consigneeAddress' 	=> $item_info->consignee,
			'shipmentID' 		=> $shipmentid,
			'packageDesc' 		=> $item_info->shipment['description'],
			'totalWeight' 		=> $item_info->shipment['weight'],
			'totalWeightUOM' 	=> $item_info->shipment['weightUom'],
			'dimensionUOM' 		=> $item_info->shipment['dimensionUom'],
			'productCode' 		=> $item_info->shipment['product_code'],
			'codValue'          => $item_info->shipment['codValue'],
			'insuranceValue' 	=> $item_info->shipment['insuranceValue'],
			'totalValue'		=> $item_info->shipment['items_value'],
			'currency' 			=> $item_info->shipment['currency'],
			'remarks'           => $item_info->shipment['remarks'],
			'shipmentPieces' 	=> array(
				array(
					'pieceID' 			=> $item_info->shipment['order_id'],
					'announcedWeight' 	=> array(
						'weight' 	=> $item_info->shipment['weight'],
						'unit' 		=> $item_info->shipment['weightUom']
					),
					'billingReference1'	=> (string)$item_info->shipment['order_id'],
					'pieceDescription'	=> $item_info->shipment['description'],
                    'codAmount'         => $item_info->shipment['codValue'],
				)
			),
		);

        // Set cross-border fields
		if( $item_info->isCrossBorder ) {
            $contents 			= $item_info->contents;
            $shipment_contents 	= array();

            foreach( $contents as $content ){

                $shipment_content = array(
                    'skuNumber' 			=> $content['sku'],
                    'description'			=> $content['description'],
                    'descriptionExport'		=> $content['descriptionExport'],
                    'itemValue' 			=> $content['value'],
                    'itemQuantity' 			=> $content['qty'],
    //				'netWeight' 			=> $content['weight'],
    //				'weightUOM' 			=> $item_info->shipment['weightUom'],
                    'countryOfOrigin' 		=> $content['origin'],
                    'hsCode'                => $content['hs_code'],
                );

                $shipment_contents[] = $shipment_content;
            }

            $shipment_item['shipmentContents'] = $shipment_contents;

            if( !empty( $item_info->shipment['incoterm'] ) ) {
                $shipment_item['incoterm'] = $item_info->shipment['incoterm'];
            }

            if( !empty( $item_info->shipment['dangerous_goods'] ) ) {
                $shipment_item['contentIndicator'] = $item_info->shipment['dangerous_goods'];
            }
		}
		
		if( !empty( $item_info->shipment['obox_service'] ) ) {
			$shipment_item['valueAddedServices']['valueAddedService'][] =
                array(
                    'vasCode' => $item_info->shipment['obox_service']
                );
		}

		$request_data = array(
			'labelRequest' 	=> array(
				'hdr' 	=> array(
					'messageType' 		=> $this->get_type(),
					'messageDateTime' 	=> $this->get_datetime(),
					'messageVersion' 	=> $this->get_version(),
					//'accessToken'		=> $token->token,
					'messageLanguage' 	=> $this->get_language()
				),
				'bd' 	=> array(
					'pickupAccountId' 	=> $this->pickup_id,
					'soldToAccountId'	=> $this->soldto_id,
					'pickupAddress' 	=> $item_info->shipper,
					'shipmentItems' 	=> array( $shipment_item ),
					'label' 			=> array(
						'format' 	=> $item_info->body[ 'label_format' ],
						'layout' 	=> $item_info->body[ 'label_layout' ],
						'pageSize' 	=> $item_info->body[ 'label_pagesize' ]
					)

				)
			)
		);

        return Args_Parser::unset_empty_values( $request_data );
	}

	protected function get_consignee( Item_Info $item_info ) {
        $consignee 			= $item_info->consignee;

        if( empty( $consignee['district'] ) && ! empty( $consignee['state'] )) {
            $consignee['district'] = $consignee['state'];
        }

        foreach ( $consignee as $consignee_key => $consignee_val ) {
            // If the field is empty do not pass it
            if( empty( $consignee_val ) ){
                unset( $consignee[ $consignee_key ] );
            }
        }

        return $consignee;
    }

	public function get_order_info($order_id) {
		$response = $this->get('orders/' . $order_id . '/');
		return $response;
	}
	public function get_merchant_info() {
		$response = $this->get('merchants/self/');
		return $response->body->data->objects[0];
	}

	// public function get_merchant_info_test_con($api_key_temp) {
	// 	$response = $this->get_test_con('couriers/', $api_key_temp);
	// 	return $response;
	// }
	
	protected function item_info_to_request_data_shipany( Item_Info $item_info ) {
		$shipmentid 		= $this->get_shipment_id( $item_info->shipment['prefix'], $item_info->shipment['order_id'] );
		// $response = $this->get('couriers/');
		$shipping_shipnay_settings = SHIPANY()->get_shipping_shipany_settings();
		$merchant = $this->get_merchant_info();

		//shipany
		$contents = $item_info->contents;
		$consignee = $this->get_consignee($item_info);
		$shipment_contents 	= array();
		foreach( $contents as $content ){
			$shipment_content = array(
				'sku' => $content['sku'],
				'name' => $content['description'],
				'unt_price' => array(
					'val' => $content['value'],
					'ccy' => $content['origin']
				),
				'qty' => $content['qty'],
				'wt' => array(
					'val' => $content["weight"]/1000,
					'unt' => 'kg'
				),
				'dim' => array(
					'len' => 1,
					'wid' => 1,
					'hgt' => 1,
					'unt' => 'cm',
				)
			);
			$shipment_contents[] = $shipment_content;
		}		
		$request_data = array(
			'cour_uid' => $item_info->shipment["product_code"],
			// 'cour_uid' => 'ad4b9127-5126-4247-9ff8-d7893ae2d2bb',
			'mch_uid' => $merchant->uid,
			'order_from' => "woocommerce",
			'ext_order_ref' => 'Order #' . $item_info->shipment["order_id"],
			'wt' => array(
				'val' => $item_info->shipment['weight']/1000,
				'unt' => 'kg'
			),
			'dim' => array(
				'len' => 1,
				'wid' => 1,
				'hgt' => 1,
				'unt' => 'cm',
			),
			'items' => $shipment_contents,
			'mch_ttl_val' => array(
				'val' => $item_info->shipment['items_value'],
				'ccy' => 'HKD'
			),
			'cour_ttl_cost' => array(
				'val' => 1,
				'ccy' => 'HKD'
			),
			'sndr_ctc' => array(
				'ctc' => array(
					'f_name' => $merchant->co_info->ctc_pers[0]->ctc->f_name,
					'l_name' => $merchant->co_info->ctc_pers[0]->ctc->l_name,
					'phs' => array(
						array(
							'typ' => $merchant->co_info->ctc_pers[0]->ctc->phs[0]->typ,
							'cnty_code' => $merchant->co_info->ctc_pers[0]->ctc->phs[0]->cnty_code,
							'num' => $merchant->co_info->ctc_pers[0]->ctc->phs[0]->num
						)
					)
				),
				'addr' => array(
					'typ' => $merchant->co_info->org_ctcs[0]->addr->typ,
					'ln' => $merchant->co_info->org_ctcs[0]->addr->ln,
					'ln2' => $merchant->co_info->org_ctcs[0]->addr->ln2,
					'city' => $merchant->co_info->org_ctcs[0]->addr->cnty,
					'cnty' => $merchant->co_info->org_ctcs[0]->addr->cnty,
					'distr' => $merchant->co_info->org_ctcs[0]->addr->distr,
					'state' => $merchant->co_info->org_ctcs[0]->addr->state
				)
			),
			'rcvr_ctc' => array(
				'ctc' => array(
					'f_name' => explode(" ", $consignee['name'])[0],
					'l_name' => explode(" ", $consignee['name'])[1],
					'phs' => array(
						array(
							'typ' => 'Mobile',
							'cnty_code' => '852',
							'num' => $consignee["phone"]
						)
					),
					'email' => $consignee['email'],
				),
				'addr' => array(
					'typ' => 'Residential',
					'ln' => $consignee['address1'],
					'ln2' => $consignee['address2'],
					'distr' => $consignee['city'],
					'cnty' => $consignee["country"],
					'state' => $consignee["state"]
				)
			)
		);

		if ($item_info->shipment["description"] != " ") {
			$request_data['mch_notes'] = array($item_info->shipment["description"]);
		}
		if ($request_data['cour_uid'] == "") {
			$request_data['cour_uid']=$shipping_shipnay_settings["shipany_default_courier"];
		}
		
		$query_rate_list = $this->query_rate($request_data);
		foreach( $query_rate_list as $query_rate ){
			if( $query_rate->cour_uid == $request_data['cour_uid'] ){
				$request_data['cour_ttl_cost'] = array('ccy' => 'HKD', 'val' => $query_rate->cour_ttl_cost->val);
				$request_data['cour_svc_pl'] = $query_rate->cour_svc_pl;
				// $request_data['cour_svc_type'] = $query_rate->cour_svc_type;
				$request_data['cour_type'] = $query_rate->cour_type;
				if (!empty($query_rate->quot_id)) {
					$request_data['quot_uid'] = $query_rate->quot_id;
				}
				break;
			}
		}

        return Args_Parser::unset_empty_values( $request_data );
	}

	/**
	 * Deletes an item from the remote API.
	 *
	 * @since [*next-version*]
	 *
	 * @param int $shipment_id The ID of the shipment to delete.
	 *
	 * @return stdClass The response.
	 *
	 * @throws Exception
	 */
	public function delete_label( $shipment_id ) {

		$route 	= $this->delete_label_route();

		$data 		= array(
			'deleteShipmentReq' 	=> array(
				'hdr' 	=> array(
					'messageType' 		=> $this->get_type( 'delete' ),
					'messageDateTime' 	=> $this->get_datetime(),
					'messageVersion' 	=> $this->get_version(),
					'messageLanguage' 	=> $this->get_language()
				),
				'bd' 	=> array(
					'pickupAccountId' 	=> $this->pickup_id,
					'soldToAccountId'	=> $this->soldto_id,
					'shipmentItems' 	=> array(
						array(
							'shipmentID' 		=> $shipment_id,
						)
					 ),
				)
			)
		);

		$response 	= $this->post($route, $data);

		$response_body = json_decode( $response->body );
		
		if ( $response->status === 200 ) {

			if( $this->check_status_code( $response_body, 'deleteShipmentResp' ) == 200 ){

				return $response_body->deleteShipmentResp->bd;

			}
		}

		throw new Exception(
			sprintf(
				__( 'Failed to delete label: %s', 'pr-shipping-shipany' ),
				$this->generate_error_details( $response_body, 'deleteShipmentResp' )
			)
		);
	}

	/**
	 * Prepares an API route with the customer namespace and EKP.
	 *
	 * @since [*next-version*]
	 *
	 * @return string
	 */
	protected function shipping_label_route() {
		//return 'rest/v2/Label';
		//shipany
		return 'orders/ ';
	}

	/**
	 * Prepares an API route for deleting label.
	 *
	 * @since [*next-version*]
	 *
	 * @return string
	 */
	protected function delete_label_route() {
		return $this->shipping_label_route(). '/Delete';
	}

	/**
	 * Prepares a CloseOut API route.
	 *
	 * @since [*next-version*]
	 *
	 * @return string
	 */
	protected function close_out_label_route() {
		return 'rest/v2/Order/Shipment/CloseOut';
	}

}
