<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}


if ( ! class_exists( 'SHIPANY_WC_Order' ) ) :

abstract class SHIPANY_WC_Order {
	const WP_POST_TIMEOUT = 30;
	
	const SHIPANY_DOWNLOAD_ENDPOINT = 'download_label';

	protected $shipping_shipnay_settings = array();

    protected $service 	= 'ShipAny';

	protected $carrier 	= '';

	/**
	 * Init and hook in the integration.
	 */
	public function __construct( ) {
		$this->define_constants();
		$this->init_hooks();

		$this->shipping_shipnay_settings = SHIPANY()->get_shipping_shipany_settings();
	}

	protected function define_constants() {
	}

	public function init_hooks() {
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ), 20 );
		add_action( 'woocommerce_process_shop_order_meta', array( $this, 'save_meta_box' ), 0, 2 );
		// Order page metabox actions
		add_action( 'wp_ajax_wc_shipment_shipany_gen_label', array( $this, 'save_meta_box_ajax' ) );
		add_action( 'wp_ajax_wc_shipment_shipany_gen_label_recreate', array( $this, 'save_meta_box_ajax_recreate' ) );
		add_action( 'wp_ajax_wc_shipment_shipany_delete_label', array( $this, 'delete_label_ajax' ) );
		add_action( 'wp_ajax_wc_send_pick_up_request',  array( $this, 'send_pickup_request' ));	
		add_action( 'woocommerce_thankyou', array( $this, 'save_meta_box_ajax_auto' ) );
		$subs_version = class_exists( 'WC_Subscriptions' ) && ! empty( WC_Subscriptions::$version ) ? WC_Subscriptions::$version : null;

		// Prevent data being copied to subscriptions
		if ( null !== $subs_version && version_compare( $subs_version, '2.0.0', '>=' ) ) {
			add_filter( 'wcs_renewal_order_meta_query', array( $this, 'woocommerce_subscriptions_renewal_order_meta_query' ), 10 );
		} else {
			add_filter( 'woocommerce_subscriptions_renewal_order_meta_query', array( $this, 'woocommerce_subscriptions_renewal_order_meta_query' ), 10 );
		}

		// display admin notices for bulk actions
		add_action( 'admin_notices', array( $this, 'render_messages' ) );

		add_action( 'init', array( $this, 'add_download_label_endpoint' ) );
		add_action( 'parse_query', array( $this, 'process_download_label' ) );
		// add {tracking_note} placeholder
		add_filter( 'woocommerce_email_format_string' , array( $this, 'add_tracking_note_email_placeholder' ), 10, 2 );
		
		add_shortcode( 'shipany_tracking_note', array( $this, 'tracking_note_shortcode') );
		add_shortcode( 'shipany_tracking_link', array( $this, 'tracking_link_shortcode') );

		
	}

	/**
	 * Add the meta box for shipment info on the order page
	 *
	 * @access public
	 */
	public function add_meta_box() {
		#shipany # Right meta box title with yellow background 
		add_meta_box( 'woocommerce-shipment-shipany-label', sprintf( __( '%s', 'pr-shipping-shipany' ), "<img src='".SHIPANY_PLUGIN_DIR_URL . "/assets/img/shipany_logo_official_banner_white_bg.png' width='170' height='50'>"), 
		array( $this, 'meta_box' ), 'shop_order', 'side', 'high' );
	}

	/**
	 * Show the meta box for shipment info on the order page
	 *
	 * @access public
	 */
	public function meta_box() {
		global $woocommerce, $post;
		
		$order_id = $post->ID;	
		// Get saved label input fields or set default values
		$shipany_label_items = $this->get_shipany_label_items( $order_id );
		// Get tracking info if it exists
		// empty( $label_tracking_info means order not create
		$label_tracking_info = $this->get_shipany_label_tracking( $order_id );
		if ( $label_tracking_info != '' ) {
			$order_detail = $this->get_order_detail($label_tracking_info["shipment_id"]);
		} else {
			$order_detail = '';
		}

		// Get saved weight, otherwise calculate it from the item weights
		if( ! empty( $shipany_label_items['shipany_weight'] ) ) {
			$selected_weight_val = $shipany_label_items['shipany_weight'];
		} else {
			$selected_weight_val = $this->calculate_order_weight( $order_id );
		}

		// Get saved product, otherwise get the default product in settings
		if( ! empty( $order_detail->cour_uid ) ) {
			$selected_courier = $order_detail->cour_uid;
		} else {
			$selected_courier = $this->get_default_courier( $order_id );
		}

		// Get the list of domestic and international services
		try {
			$shipany_obj = SHIPANY()->get_shipany_factory();
			$shipany_product_list = $shipany_obj->get_shipany_products_international();

		} catch (Exception $e) {

			echo '<p class="wc_shipany_error">' . esc_html($e->getMessage()) . '</p>';
		}

		$delete_label = '';
		if ($this->can_delete_label($order_id)) {
			$delete_label = '<span class="wc_shipany_delete"><a href="#" id="shipany_delete_label">' . __('Delete Label', 'pr-shipping-shipany') . '</a></span>';
		}

		//shipany
		//$main_button = '<button id="shipany-label-button" class="button button-primary button-save-form">' . __( 'Generate Label', 'pr-shipping-shipany' ) . '</button>';
		$main_button = '<button id="shipany-label-button" class="button button-primary button-save-form">' . __( 'Create ShipAny Order', 'pr-shipping-shipany' ) . '</button>';


		// Get tracking info if it exists
		// empty( $label_tracking_info means order not create
		// $label_tracking_info = $this->get_shipany_label_tracking( $order_id );
		// if ( $label_tracking_info != '' ) {
		// 	$order_detail = $this->get_order_detail($label_tracking_info["shipment_id"]);
		// } else {
		// 	$order_detail = '';
		// }
		if ( $order_detail == '' || $order_detail->pay_stat != "Insufficient balance") {
			if ($order_detail !='') {
				echo '<div id="order_status">Order Status: '. $order_detail->cur_stat . '</div>';
			}
			
		
			// Check whether the label has already been created or not
			if( empty( $label_tracking_info ) ) {
				$is_disabled = '';
				
				$print_button = '<a href="#" id="shipany-label-print" class="button button-primary" download target="_blank">' . __( 'Download Label', 'pr-shipping-shipany' ) . '</a>';

			} else {
				$is_disabled = 'disabled';
				if ($order_detail->lab_url == '') {
					$print_button = '<a href="'. $this->get_download_label_url( $order_id ) .'" id="shipany-label-print" class="button button-primary" download target="_blank" disabled="disabled" style="pointer-events: none">' .__( 'Download Label', 'pr-shipping-shipany' ) . '</a>';
				} else {
					$print_button = '<a href="'. get_site_url().'/wp-content/uploads/woocommerce_shipany_label/shipany-'.$label_tracking_info['shipment_id'].'.pdf' .'" id="shipany-label-print" class="button button-primary" download target="_blank">' .__( 'Download Label', 'pr-shipping-shipany' ) . '</a>';
				}
				
			}

			if( empty( $label_tracking_info ) ) {
				$is_disabled = '';
				
				$pickup_request = '<button id="shipany-pickup-button" class="button button-primary button-save-form">' . __( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</button>';

			} else {
				$is_disabled = 'disabled';

				// $pickup_request = '<a href="'. $this->send_pickup_request( $label_tracking_info ) .'" id="send_pickup_request" class="button button-primary">' .__( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</a>';
				$pickup_request = '<button id="shipany-pickup-button" class="button button-primary button-save-form">' . __( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</button>';
			}

			// when load page, get the order detail, if 
			if (!empty($label_tracking_info)) {
				if ($order_detail->cur_stat == 'Pickup Request Sent' || $order_detail->cur_stat == 'Pickup Request Received') {
					$pickup_request = '<button id="shipany-pickup-button" disabled="disabled" class="button button-primary button-save-form">' . __( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</button>';
				}
			}


			$shipany_label_data = array(
				'main_button' => $main_button,
				'delete_label' => $delete_label,
				'print_button' => $print_button,
				'pickup_request' => $pickup_request
			);


			echo '<div id="shipment-shipany-label-form">';

			if( !empty( $shipany_product_list ) ) {
			

				woocommerce_wp_hidden_input( array(
					'id'    => 'pr_shipany_label_nonce',
					'value' => wp_create_nonce( 'create-shipany-label' )
				) );

				woocommerce_wp_select ( array(
					'id'          		=> 'pr_shipany_product',
					'label'       		=> __( 'Courier selected:', 'pr-shipping-shipany' ), #shipany #rename button 
					'description'		=> '',
					'value'       		=> $selected_courier,
					'options'			=> $shipany_product_list,
					'custom_attributes'	=> array( $is_disabled => $is_disabled )
				) );

				$weight_units = get_option( 'woocommerce_weight_unit' );
				// Get weight UoM and add in label
				woocommerce_wp_text_input( array(
					'id'          		=> 'shipany_weight',
					'label'       		=> sprintf( __( 'Estimated shipment weight (%s) based on items ordered: ', 'pr-shipping-shipany' ), $weight_units),
					'placeholder' 		=> 'Please enter package weight',
					'description'		=> '',
					'value'       		=> $selected_weight_val,
					'custom_attributes'	=> array( $is_disabled => $is_disabled ),
					'class'				=> 'wc_input_decimal' // adds JS to validate input is in price format
				) );

				$this->additional_meta_box_fields( $order_id, $is_disabled, $shipany_label_items, $shipany_obj );


				// A label has been generated already, allow to delete
				if( empty( $label_tracking_info ) ) {
					echo wp_kses_post($main_button);
				} else {
					echo $print_button;
					echo "<p>";
					echo wp_kses_post($pickup_request);
				}
				
				wp_enqueue_script( 'wc-shipment-shipany-label-js', SHIPANY_PLUGIN_DIR_URL . '/assets/js/shipany.js', array('jquery'), SHIPANY_VERSION );
				wp_localize_script( 'wc-shipment-shipany-label-js', 'shipany_label_data', $shipany_label_data );
				
			} else {
				echo wp_kses_post('<p class="wc_shipany_error">' . __('There are no services available for the destination country!', 'pr-shipping-shipany') . '</p>');
			}
			
			echo '</div>';
		# handle out of credit condition.
		} else if ($order_detail != '' && $order_detail->pay_stat == "Insufficient balance") {

			?>
			<style type="text/css">
				/* button.button-primary.recreate_button{
				background: red !important;
				} */

				button.button-primary.button-save-form.tooltip {
				position: relative;
				}
				button.button-primary.button-save-form.tooltip::before {
				background-color: #a7cedc;
				border: 1px solid #888;
				border-radius: 2px;
				color: #000;
				content: attr(data-title);
				display: none;
				font-family: sans-serif;
				font-size: 10px;
				padding: 2px 5px;
				position: absolute;
				top: -30px;
				left: -30px;
				z-index: 1;
				}
				button.button-primary.button-save-form.tooltip:hover::before {
				display: block;
				}
				
			</style>
			<?php
			// Check whether the label has already been created or not
			$main_button = '<button id="shipany-label-button-recreate" class="button button-primary button-save-form tooltip" data-title="Since ShipAny account is out of credit, the order falls to draft state">' . __( 'Create ShipAny Order (From Draft)', 'pr-shipping-shipany' ) . '</button>';
			if( empty( $label_tracking_info ) ) {
				$is_disabled = '';
				
				$print_button = '<a href="#" id="shipany-label-print" class="button button-primary" download target="_blank">' . __( 'Download Label', 'pr-shipping-shipany' ) . '</a>';

			} else {
				$is_disabled = 'disabled';

				if ($order_detail->lab_url == '') {
					$print_button = '<a href="'. $this->get_download_label_url( $order_id ) .'" id="shipany-label-print" class="button button-primary" download target="_blank" disabled="disabled" style="pointer-events: none">' .__( 'Download Label', 'pr-shipping-shipany' ) . '</a>';
				} else {
					$print_button = '<a href="'. get_site_url().'/wp-content/uploads/woocommerce_shipany_label/shipany-'.$label_tracking_info['shipment_id'].'.pdf' .'" id="shipany-label-print" class="button button-primary" download target="_blank">' .__( 'Download Label', 'pr-shipping-shipany' ) . '</a>';
				}
				
			}

			if( empty( $label_tracking_info ) ) {
				$is_disabled = '';
				
				$pickup_request = '<button id="shipany-pickup-button" class="button button-primary button-save-form">' . __( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</button>';

			} else {
				$is_disabled = 'disabled';

				// $pickup_request = '<a href="'. $this->send_pickup_request( $label_tracking_info ) .'" id="send_pickup_request" class="button button-primary">' .__( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</a>';
				$pickup_request = '<button id="shipany-pickup-button" class="button button-primary button-save-form">' . __( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</button>';
			}

			// when load page, get the order detail, if 
			if (!empty($label_tracking_info)) {
				if ($order_detail->cur_stat == 'Pickup Request Sent' || $order_detail->cur_stat == 'Pickup Request Received') {
					$pickup_request = '<button id="shipany-pickup-button" disabled="disabled" class="button button-primary button-save-form">' . __( 'Send Pickup Request', 'pr-shipping-shipany' ) . '</button>';
				}
			}


			$shipany_label_data = array(
				'main_button' => $main_button,
				'delete_label' => $delete_label,
				'print_button' => $print_button,
				'pickup_request' => $pickup_request,
				'label_tracking_info' => $label_tracking_info,
				'mch_uid' => $order_detail->mch_uid
			);


			echo '<div id="shipment-shipany-label-form">';

			if( !empty( $shipany_product_list ) ) {

				echo '<div id="order_status">Order Status: '. $order_detail->cur_stat . '</div>';

				woocommerce_wp_hidden_input( array(
					'id'    => 'pr_shipany_label_nonce',
					'value' => wp_create_nonce( 'create-shipany-label' )
				) );

				woocommerce_wp_select ( array(
					'id'          		=> 'pr_shipany_product',
					'label'       		=> __( 'Courier selected:', 'pr-shipping-shipany' ), #shipany #rename button 
					'description'		=> '',
					'value'       		=> $selected_courier,
					'options'			=> $shipany_product_list,
					'custom_attributes'	=> array( $is_disabled => $is_disabled )
				) );

				$weight_units = get_option( 'woocommerce_weight_unit' );
				// Get weight UoM and add in label
				woocommerce_wp_text_input( array(
					'id'          		=> 'shipany_weight',
					'label'       		=> sprintf( __( 'Estimated shipment weight (%s) based on items ordered: ', 'pr-shipping-shipany' ), $weight_units),
					'placeholder' 		=> 'Please enter package weight',
					'description'		=> '',
					'value'       		=> $selected_weight_val,
					'custom_attributes'	=> array( $is_disabled => $is_disabled ),
					'class'				=> 'wc_input_decimal' // adds JS to validate input is in price format
				) );

				$this->additional_meta_box_fields( $order_id, $is_disabled, $shipany_label_items, $shipany_obj );


				// A label has been generated already, allow to delete
				// if( empty( $label_tracking_info ) ) {
				// 	echo wp_kses_post($main_button);
				// } else {
				// 	echo $print_button;
				// 	echo "<p>";
				// 	echo wp_kses_post($pickup_request);
				// }
				echo wp_kses_post($main_button);
				
				wp_enqueue_script( 'wc-shipment-shipany-label-js', SHIPANY_PLUGIN_DIR_URL . '/assets/js/shipany.js', array('jquery'), SHIPANY_VERSION );
				wp_localize_script( 'wc-shipment-shipany-label-js', 'shipany_label_data', $shipany_label_data );
				
			} else {
				echo wp_kses_post('<p class="wc_shipany_error">' . __('There are no services available for the destination country!', 'pr-shipping-shipany') . '</p>');
			}
			
			echo '</div>';
		}
		
	}
	
	public function get_order_detail($order_id) {
		$response = SHIPANY()->get_shipany_factory()->api_client->get_order_info($order_id);
		if (!empty($response->body->data->objects[0]->cur_stat)) {
			return $response->body->data->objects[0];
		}
		return '';
	}

	protected function can_delete_label($order_id) {
		return true;
	}

	abstract public function additional_meta_box_fields( $order_id, $is_disabled, $shipany_label_items, $shipany_obj );


	public function save_meta_box( $post_id, $post = null ) {
		// loop through inputs within id 'shipment-shipany-label-form'
		$meta_box_ids = array( 'pr_shipany_product', 'shipany_weight');

		$additional_meta_box_ids = $this->get_additional_meta_ids( );
		$meta_box_ids = array_merge( $meta_box_ids, $additional_meta_box_ids );
		foreach ($meta_box_ids as $key => $value) {
			// Save value if it exists
			if ( isset( $_POST[ $value ] ) ) {
				$args[ $value ]	 = wc_clean( $_POST[ $value ] );
			} else {
                $args[ $value ]	 = '';
            }
		}

		if( isset( $args ) ) {
			$this->save_shipany_label_items( $post_id, $args );
			return $args;
		}
	}
	
	abstract public function get_additional_meta_ids();
	/**
	 * Order Tracking Save AJAX
	 *
	 * Function for saving tracking items
	 */
	public function save_meta_box_ajax_recreate( ) {
		$order_id = wc_clean( $_POST[ 'order_id' ] );
		// Save inputted data first
		$this->save_meta_box( $order_id );

		$api_tk = $this->shipping_shipnay_settings["shipany_api_key"];
		$temp_api_endpoint = 'https://api.shipany.io/';
		//dev3
        if (strpos($api_tk, 'SHIPANYDEV') !== false){
            $temp_api_endpoint = 'https://api-dev3.shipany.io/';
            $api_tk = str_replace("SHIPANYDEV", "", $api_tk);
        }
		//demo1
        if (strpos($api_tk, 'SHIPANYDEMO') !== false){
            $temp_api_endpoint = 'https://api-demo1.shipany.io/';
            $api_tk = str_replace("SHIPANYDEMO", "", $api_tk);
        }	
		
		$myObj = new stdClass();
		$myObj->mch_uid = wc_clean( $_POST[ 'mch_id' ] );
		if ($myObj->mch_uid ==''){
			$response = wp_remote_get($temp_api_endpoint.'merchants/self/', array(
				'headers' => array(
					'api-tk'=> $api_tk
				)
			));
			if ($response["response"]["code"] == 200) {
				$myObj->mch_uid = json_decode($response['body'])->data->objects[0]->uid;
			}
		}
		$myObj->uid = wc_clean( $_POST[ 'shipany_order_id' ] );
		if ($myObj->uid == ''){
			$myObj->uid = $this->get_shipany_label_tracking( $order_id)['shipment_id'];
		}
		$myJSON = json_encode($myObj);


		$wp_shipany_rest_response = wp_remote_post(
		    $temp_api_endpoint . 'orders/charge-order/ ',
		    array( 'headers' => array('api-tk'=> $api_tk),
		    		'body' => $myJSON,
					'timeout' => self::WP_POST_TIMEOUT)
		);
		# Still no enough credit
		if ($wp_shipany_rest_response["response"]["code"] != 200) {
			wp_send_json( array(
				'error' => 'error'
				) );
			
			wp_die();
		} else if ($wp_shipany_rest_response["response"]["code"] == 200) {


			$label_tracking_info = $this->get_shipany_label_tracking( $order_id );
			$order_detail = $this->get_order_detail($label_tracking_info["shipment_id"]);


			$label_url = $order_detail->lab_url;
			$response= wp_remote_get($label_url, array( 'sslverify' => false ));
			$label_pdf_data = wp_remote_retrieve_body( $response );
			file_put_contents( SHIPANY()->get_shipany_label_folder_dir() . 'shipany-' . $label_tracking_info["shipment_id"] . '.pdf', $label_pdf_data );

			$tracking_note = $this->get_tracking_note( $order_id );
			$tracking_note_type = $this->get_tracking_note_type();
			$get_file_size = filesize($label_tracking_info["label_path"]);
			$insufficient_balance = $label_tracking_info["insufficient_balance"];

			wp_send_json( array(
				'button_txt' => __( 'Download Label', 'pr-shipping-shipany' ),
				'label_url' => get_site_url().'/wp-content/uploads/woocommerce_shipany_label/shipany-'.$label_tracking_info['shipment_id'].'.pdf',
				'get_file_size' => $get_file_size,
				'insufficient_balance' => $insufficient_balance,
				'courier_id' => $order_detail->cour_uid
				) );
			
			wp_die();
		}
	
	}
	public function save_meta_box_ajax( ) {
		check_ajax_referer( 'create-shipany-label', 'pr_shipany_label_nonce' );
		$order_id = wc_clean( $_POST[ 'order_id' ] );

		// Save inputted data first
		$this->save_meta_box( $order_id );

		// Gather args for API call
		$args = $this->get_label_args( $order_id );

		// Allow third parties to modify the args to the APIs
		$args = apply_filters('shipping_shipany_label_args', $args, $order_id );

		$shipany_obj = SHIPANY()->get_shipany_factory();
		$label_tracking_info = $shipany_obj->get_shipany_label( $args );

		$this->save_shipany_label_tracking( $order_id, $label_tracking_info );
		$tracking_note = $this->get_tracking_note( $order_id );
		$tracking_note_type = $this->get_tracking_note_type();
		$label_url = $this->get_download_label_url( $order_id );

		//check 0 size pdf
		$get_file_size = filesize($label_tracking_info["label_path"]);
		$insufficient_balance = $label_tracking_info["insufficient_balance"];
		
		do_action( 'pr_shipping_shipany_label_created', $order_id );

		wp_send_json( array( 
			'download_msg' => __('Your label is ready to download, click the "Download Label" button above"', 'pr-shipping-shipany'),
			'button_txt' => __( 'Download Label', 'pr-shipping-shipany' ),
			'label_url' => get_site_url().'/wp-content/uploads/woocommerce_shipany_label/shipany-'.$label_tracking_info['shipment_id'].'.pdf',
			'tracking_note'	  => $tracking_note,
			'tracking_note_type' => $tracking_note_type,
			'get_file_size' => $get_file_size,
			'insufficient_balance' => $insufficient_balance
			) );
		
		wp_die();
	}
	public function save_meta_box_ajax_auto($order_id ) {
		// $order_id = wc_clean( $_POST[ 'order_id' ] );
		// Save inputted data first
		if ($this->shipping_shipnay_settings['set_default_create'] == 'no') {
			return;
		}
		$this->save_meta_box( $order_id );

		// Gather args for API call
		$args = $this->get_label_args( $order_id );

		// Allow third parties to modify the args to the APIs
		$args = apply_filters('shipping_shipany_label_args', $args, $order_id );
		// $args['order_details']['weight']=$args['items'][0]['item_weight'] * $args['items'][0]['qty'];
		foreach ($args["items"] as $element) {
			$args['order_details']['weight'] = intval($args['order_details']['weight']) + ($element['qty'] * intval($element['item_weight']));
		}
		$args['order_details']['weight'] = strval($args['order_details']['weight']);
		$shipany_obj = SHIPANY()->get_shipany_factory();
		$label_tracking_info = $shipany_obj->get_shipany_label( $args );

		$this->save_shipany_label_tracking( $order_id, $label_tracking_info );
		$tracking_note = $this->get_tracking_note( $order_id );
		$tracking_note_type = $this->get_tracking_note_type();
		$label_url = $this->get_download_label_url( $order_id );

		do_action( 'pr_shipping_shipany_label_created', $order_id );

		$order = wc_get_order( $order_id );
		$order->add_order_note( $tracking_note, $tracking_note_type, true );
		// wp_send_json( array( 
		// 	'download_msg' => __('Your label is ready to download, click the "Download Label" button above"', 'pr-shipping-shipany'),
		// 	'button_txt' => __( 'Download Label', 'pr-shipping-shipany' ),
		// 	'label_url' => $label_url,
		// 	'tracking_note'	  => $tracking_note,
		// 	'tracking_note_type' => $tracking_note_type,
		// 	'get_file_size' => $get_file_size,
		// 	'insufficient_balance' => $insufficient_balance
		// 	) );
		
		// wp_die();
	}
	public function delete_label_ajax( ) {
		check_ajax_referer( 'create-shipany-label', 'pr_shipany_label_nonce' );
		$order_id = wc_clean( $_POST[ 'order_id' ] );

		try {

			$args = $this->delete_label_args( $order_id );
			$shipany_obj = SHIPANY()->get_shipany_factory();
			
			// Delete meta data first in case there is an error with the API call
			$this->delete_shipany_label_tracking( $order_id ); 
			$shipany_obj->delete_shipany_label( $args );
			
			$tracking_num = $args['tracking_number'];

			wp_send_json( array( 
				'download_msg' => __('Your label is ready to download, click the "Download Label" button above"', 'pr-shipping-shipany'), 
				//shipany
				//'button_txt' => __( 'Generate Label', 'pr-shipping-shipany' ), 
				'button_txt' => __( 'Create ShipAny Order', 'pr-shipping-shipany' ), 
				'shipany_tracking_num'	  => $tracking_num
				) );

		} catch (Exception $e) {

			wp_send_json( array( 'error' => $e->getMessage() ) );
		}
	}

	public function send_pickup_request( ) {
		$order_id = wc_clean( $_POST[ 'order_id' ] );
		$args = $this->get_shipany_label_tracking( $order_id );
		// $shipany_obj = SHIPANY()->get_shipany_factory();
		$pickup_request_result = SHIPANY()->get_shipany_factory()->send_pickup_request( $args["tracking_number"]);
		$lab_url = $pickup_request_result->body->data->objects[0]->lab_url;
		if ($pickup_request_result->body->data->objects[0]->cour_api_typ == 'Zeek2Door') {
			$lab_url = $pickup_request_result->body->data->objects[0]->lab_url;
			$response= wp_remote_get($lab_url, array( 'sslverify' => false ));
			$label_pdf_data = wp_remote_retrieve_body( $response );
			$shipment_id = $pickup_request_result->body->data->objects[0]->uid;
			$shipany_obj = new SHIPANY_API_eCS_Asia( 'dum' );
			$shipany_obj->save_shipany_label_file( 'item', $shipment_id, $label_pdf_data );
			wp_send_json( array( 
				'lab_url' => get_site_url().'/wp-content/uploads/woocommerce_shipany_label/shipany-'.$shipment_id.'.pdf'
				) );
		}

		// wp_send_json( array( 
		// 	'lab_url'	  => $lab_url,
		// 	'val'		  => $pickup_request_result->body->data->objects[0]
		// 	) );
		// wp_die();
		// if ($pickup_request_result->status == 200) {
		// }

		// $label_tracking_info = $this->get_shipany_label_tracking( $order_id );
		// // Check whether the label has already been created or not
		// if( empty( $label_tracking_info ) ) {
		// 	return '';
		// }
		
		// // If no 'label_path' isset but a 'label_url' is set them return it...
		// // ... this indicates an old download style label!
		// if ( ! isset( $label_tracking_info['label_path'] ) && isset( $label_tracking_info['label_url'] ) ){
		// 	return $label_tracking_info['label_url'];
		// }

		// // Override URL with our solution's download label endpoint:
		// return $this->generate_download_url( '/' . self::SHIPANY_DOWNLOAD_ENDPOINT . '/' . $order_id );
	}

	protected function get_download_label_url( $order_id ) {
		
		if( empty( $order_id ) ) {
			return '';
		}

		$label_tracking_info = $this->get_shipany_label_tracking( $order_id );
		// Check whether the label has already been created or not
		if( empty( $label_tracking_info ) ) {
			return '';
		}
		
		// If no 'label_path' isset but a 'label_url' is set them return it...
		// ... this indicates an old download style label!
		if ( ! isset( $label_tracking_info['label_path'] ) && isset( $label_tracking_info['label_url'] ) ){
			return $label_tracking_info['label_url'];
		}

		// Override URL with our solution's download label endpoint:
		return $this->generate_download_url( '/' . self::SHIPANY_DOWNLOAD_ENDPOINT . '/' . $order_id );
	}

	protected function get_tracking_note( $order_id ) {
		
		if( ! empty( $this->shipping_shipnay_settings['shipany_tracking_note_txt'] ) ) {
			$tracking_note = $this->shipping_shipnay_settings['shipany_tracking_note_txt'];
		} else {
			$tracking_note = sprintf( __( 'ShipAny Tracking Number: {tracking-link}', 'pr-shipping-shipany' ));
		}
		
		$tracking_link = $this->get_tracking_link( $order_id );

		if( empty( $tracking_link ) ) {
		    return '';
        }

		$tracking_note_new = str_replace('{tracking-link}', $tracking_link, $tracking_note, $count);
		if( $count == 0 ) {
			$tracking_note_new = $tracking_note . ' ' . $tracking_link;
		}
		
		return $tracking_note_new;
	}

	protected function get_tracking_link( $order_id ) {
		
		$label_tracking_info = $this->get_shipany_label_tracking( $order_id );
		if( empty( $label_tracking_info['tracking_number'] ) ) {
			return '';
		}

		return sprintf( __( '<a href="%s%s" target="_blank">%s</a>', 'pr-shipping-shipany' ), $this->get_tracking_url(), $label_tracking_info['tracking_number'], $label_tracking_info['tracking_number']);
	}

	abstract protected function get_tracking_url();

	protected function get_tracking_note_type() {
		if( isset( $this->shipping_shipnay_settings['shipany_tracking_note'] ) && ( $this->shipping_shipnay_settings['shipany_tracking_note'] == 'yes' ) ) {
			return '';
		} else {
			return 'customer';
		}
	}

	public function add_tracking_note_email_placeholder( $string, $email ) {

		$placeholder = '{shipany_tracking_note}'; // The corresponding placeholder to be used
		
    	$order = $email->object; // Get the instance of the WC_Order Object
		
		// Ensure the object is an order and not another type
		if ( ! ( $order instanceof WC_Order ) ) {
    		return $string;
    	}

		$tracking_note = $this->get_tracking_note( $order->get_id() );

    	// Return the clean replacement tracking_note string for "{tracking_note}" placeholder
    	return str_replace( $placeholder, $tracking_note, $string );
	}
	
	public function tracking_note_shortcode( $atts, $content ) {

		extract(shortcode_atts(array(
			'order_id' => ''
		), $atts));

		if( $order = wc_get_order( $order_id ) ){

			return $this->get_tracking_note( $order->get_id() );

		}

    	return '';
	}

	public function tracking_link_shortcode( $atts, $content ) {

		extract(shortcode_atts(array(
			'order_id' => ''
		), $atts));

		if( $order = wc_get_order( $order_id ) ){

			return $this->get_tracking_link( $order->get_id() );

		}

    	return '';
	}

	/**
	 * Saves the tracking items array to post_meta.
	 *
	 * @param int   $order_id       Order ID
	 * @param array $tracking_items List of tracking item
	 *
	 * @return void
	 */
	public function save_shipany_label_tracking( $order_id, $tracking_items ) {

		if( isset( $tracking_items['label_path'] ) && validate_file( $tracking_items['label_path'] ) === 2 ){
			$tracking_items['label_path'] = wp_slash( $tracking_items['label_path'] );
		}
		
		update_post_meta( $order_id, '_pr_shipment_shipany_label_tracking', $tracking_items );

		$tracking_details = array(
			'carrier' 			=> $this->carrier,
			'tracking_number' 	=> $tracking_items['tracking_number'],
			'ship_date' 		=> date( "Y-m-d", time() )
		);

		// Primarily added for "Advanced Tracking" plugin integration
		do_action( 'pr_save_shipany_label_tracking', $order_id, $tracking_details );
	}

	/*
	 * Gets all tracking items fron the post meta array for an order
	 *
	 * @param int  $order_id  Order ID
	 *
	 * @return tracking items
	 */
	public function get_shipany_label_tracking( $order_id ) {
		return get_post_meta( $order_id, '_pr_shipment_shipany_label_tracking', true );
	}

	/**
	 * Delete the tracking items array to post_meta.
	 *
	 * @param int   $order_id       Order ID
	 *
	 * @return void
	 */
	public function delete_shipany_label_tracking( $order_id ) {
		delete_post_meta( $order_id, '_pr_shipment_shipany_label_tracking' );

		do_action( 'pr_delete_shipany_label_tracking', $order_id );
	}

	/**
	 * Saves the label items array to post_meta.
	 *
	 * @param int   $order_id       Order ID
	 * @param array $tracking_items List of tracking item
	 *
	 * @return void
	 */
	public function save_shipany_label_items( $order_id, $tracking_items ) {
		update_post_meta( $order_id, '_pr_shipment_shipany_label_items', $tracking_items );
	}

	/*
	 * Gets all label items fron the post meta array for an order
	 *
	 * @param int  $order_id  Order ID
	 *
	 * @return label items
	 */
	public function get_shipany_label_items( $order_id ) {
		return get_post_meta( $order_id, '_pr_shipment_shipany_label_items', true );
	}

	/*
	 * Save default fields, used by bulk create label
	 *
	 * @param int  $order_id  Order ID
	 *
	 * @return default label items
	 */
	protected function save_default_shipany_label_items( $order_id ) {
		$shipany_label_items = $this->get_shipany_label_items( $order_id );

		if( empty( $shipany_label_items ) ) {
			$shipany_label_items = array();
		}

		if( empty( $shipany_label_items['shipany_weight'] ) ) {
			// Set default weight
			$shipany_label_items['shipany_weight'] = $this->calculate_order_weight( $order_id );
		}

		// if( empty( $shipany_label_items['pr_shipany_product'] ) ) {
		// 	// Set default product
		// 	$shipany_label_items['pr_shipany_product'] = $this->get_default_courier( $order_id );
		// }

		// Save default items
		$this->save_shipany_label_items( $order_id, $shipany_label_items );
	}

	protected function get_default_courier( $order_id ) {
		// $this->shipping_shipnay_settings = SHIPANY()->get_shipping_shipany_settings();
		return $this->shipping_shipnay_settings['shipany_default_courier'];
	}

	protected function calculate_order_weight( $order_id ) {
		
		$total_weight 	= 0;
		$order 			= wc_get_order( $order_id );

		if( false === $order ){
			return apply_filters('shipping_shipany_order_weight', $total_weight, $order_id );	
		}

		$ordered_items = $order->get_items( );

		if( is_array( $ordered_items ) && count( $ordered_items ) > 0 ){

			foreach ($ordered_items as $key => $item) {
					
				if( ! empty( $item['variation_id'] ) ) {
					$product = wc_get_product($item['variation_id']);
				} else {
					$product = wc_get_product( $item['product_id'] );
				}
				
				if ( $product ) {
					$product_weight = $product->get_weight();
					if( $product_weight ) {
						$total_weight += ( $item['qty'] * $product_weight );
					}
				}
			}

		}

		if ( ! empty( $this->shipping_shipnay_settings['shipany_add_weight'] ) ) {

			if ( $this->shipping_shipnay_settings['shipany_add_weight_type'] == 'absolute' ) {
				$total_weight += $this->shipping_shipnay_settings['shipany_add_weight'];
			} elseif ( $this->shipping_shipnay_settings['shipany_add_weight_type'] == 'percentage' ) {
				$total_weight += $total_weight * ( $this->shipping_shipnay_settings['shipany_add_weight'] / 100 );
			}
		}

		return apply_filters('shipping_shipany_order_weight', $total_weight, $order_id );
	}

	protected function is_shipping_domestic( $order_id ) {   	 
		$order = wc_get_order( $order_id );
		$shipping_address = $order->get_address( 'shipping' );
		$shipping_country = $shipping_address['country'];

		if( SHIPANY()->is_shipping_domestic( $shipping_country ) ) {
			return true;
		} else {
			return false;
		}
	}

	protected function is_crossborder_shipment( $order_id ) {   	 
		$order = wc_get_order( $order_id );
		$shipping_address = $order->get_address( 'shipping' );
		$shipping_country = $shipping_address['country'];

		if( SHIPANY()->is_crossborder_shipment( $shipping_country ) ) {
			return true;
		} else {
			return false;
		}
	}

	// This function gathers all of the data from WC to send to API
	public function get_label_args( $order_id ) {

		$shipany_label_items = $this->get_shipany_label_items( $order_id );

		if($shipany_label_items["shipany_description"]==""){
			$shipany_label_items["shipany_description"] = " ";
		}
		// Get settings from child implementation
		$args = $this->get_label_args_settings( $order_id, $shipany_label_items );		
		
		$order = wc_get_order( $order_id );
		// Get service product
		$args['order_details']['shipany_product'] = $shipany_label_items['pr_shipany_product'];
		// $args['order_details']['duties'] = $shipany_label_items['shipping_shipany_duties'];
		$args['order_details']['weight'] = $shipany_label_items['shipany_weight'];

		// Get WC specific details; order id, currency, units of measure, COD amount (if COD used)
		$args['order_details']['order_id'] = $order_id;
		// $args['order_details']['currency'] = get_woocommerce_currency();
		$args['order_details']['currency'] = $this->get_wc_currency( $order_id );
		$weight_units = get_option( 'woocommerce_weight_unit' );
		
		switch ( $weight_units ) {
			case 'lbs':
				$args['order_details']['weightUom'] = 'lb';
				break;
			default:
				$args['order_details']['weightUom'] = $weight_units;
				break;
		}

		$args['order_details']['dimUom'] = get_option( 'woocommerce_dimension_unit' );

		if( $this->is_cod_payment_method( $order_id ) ) {
			$args['order_details']['cod_value']	= $order->get_total();			
		}

		// calculate the additional fee
		$additional_fees = 0;
		if( count( $order->get_fees() ) > 0 ){
			foreach( $order->get_fees() as $fee ){

				if( defined( 'WOOCOMMERCE_VERSION' ) && version_compare( WOOCOMMERCE_VERSION, '3.0.0', '>=' ) ){

					$additional_fees += floatval( $fee->get_total() );

				}else{
					
					$additional_fees += floatval( $fee['line_total'] );
					
				}
				
			}
		}

		$args['order_details']['additional_fee'] 	= $additional_fees;

		if( defined( 'WOOCOMMERCE_VERSION' ) && version_compare( WOOCOMMERCE_VERSION, '3.0.0', '>=' ) ){

			$args['order_details']['shipping_fee'] 		= $order->get_shipping_total();

		}else{

			$args['order_details']['shipping_fee'] 		= $order->get_total_shipping();

		}
		
		
		$args['order_details']['total_value'] = $order->get_total();			
		
		// Get address related information 
		$billing_address = $order->get_address();
		$shipping_address = $order->get_address( 'shipping' );

		// If shipping phone number doesn't exist, try to get billing phone number
		// if( ! isset( $shipping_address['phone'] ) && isset( $billing_address['phone'] ) ) {
		if( isset( $billing_address['phone'] ) ) {
			$shipping_address['phone'] = $billing_address['phone'];			
		}
		
		// If shipping email doesn't exist, try to get billing email
		if( ! isset( $shipping_address['email'] ) && isset( $billing_address['email'] ) ) {
			$shipping_address['email'] = $billing_address['email'];
		}

		// Merge first and last name into "name"
		$shipping_address['name'] = '';
		if ( isset( $shipping_address['first_name'] ) ) {
			$shipping_address['name'] = $shipping_address['first_name'];
			// unset( $shipping_address['first_name'] );
		}

		if ( isset( $shipping_address['last_name'] ) ) {
			if( ! empty( $shipping_address['name'] ) ) {
				$shipping_address['name'] .= ' ';
			}

			$shipping_address['name'] .= $shipping_address['last_name'];
			// unset( $shipping_address['last_name'] );
		}
		
		// If not USA or Australia, then change state from ISO code to name
		if ( $shipping_address['country'] != 'US' && $shipping_address['country'] != 'AU' ) {
			// Get all states for a country
			$states = WC()->countries->get_states( $shipping_address['country'] );

			// If the state is empty, it was entered as free text
			if ( ! empty($states) && ! empty( $shipping_address['state'] ) ) {
				// Change the state to be the name and not the code
				$shipping_address['state'] = $states[ $shipping_address['state'] ];
				
				// Remove anything in parentheses (e.g. TH)
				$ind = strpos($shipping_address['state'], " (");
				if( false !== $ind ) {
					$shipping_address['state'] = substr( $shipping_address['state'], 0, $ind );
				}
			}
		}

		// Check if post number exists then send over
		if( $shipping_shipany_postnum = get_post_meta( $order_id, '_shipping_shipany_postnum', true ) ) {
			$shipping_address['shipany_postnum'] = $shipping_shipany_postnum;
		}

		$args['shipping_address'] = $shipping_address;

		// Get order item specific data
		$ordered_items = $order->get_items( );
		$args['items'] = array();
		// Sum value of ordered items
		$args['order_details']['items_value'] = 0;
		foreach ($ordered_items as $key => $item) {
            // Reset array
            $new_item = array();

			$new_item['qty'] = $item['qty'];
			// Get 1 item value not total items, based on ordered items in case currency is different that set product price
			$new_item['item_value'] = ( $item['line_total'] / $item['qty'] );
			// Sum 'line_total' to get items total value w/ discounts!
			$args['order_details']['items_value'] += $item['line_total'];

			$product = wc_get_product( $item['product_id'] );

			// If product does not exist (i.e. was deleted) OR is virtual, skip it
			if ( empty( $product ) || $product->is_virtual() ) {
				continue;
			}

		    $country_value = get_post_meta( $item['product_id'], '_shipany_manufacture_country', true );
		    if( ! empty( $country_value ) ) {
		    	$new_item['country_origin'] = $country_value;
		    }

			$hs_code = get_post_meta( $item['product_id'], '_shipany_hs_code', true );
			if( ! empty( $hs_code ) ) {
				$new_item['hs_code'] = $hs_code;
			}

			$new_item['item_description'] = $product->get_title();
			// $new_item['line_total'] = $item['line_total'];

			if( ! empty( $item['variation_id'] ) ) {
				$product_variation = wc_get_product($item['variation_id']);

				// If product variation does not exist (i.e. was deleted) OR is virtual, skip it
				if ( empty( $product_variation ) || $product_variation->is_virtual() ) {
					continue;
				}

				// place 'sku' in a variable before validating using 'empty' to be compatible with PHP v5.4
				$product_sku = $product_variation->get_sku();
				// Ensure id is string and not int
				$new_item['product_id'] = intval( $item['variation_id'] );
				$new_item['sku'] = empty( $product_sku ) ? strval( $item['variation_id'] ) : $product_sku;

				// If value is empty due to discounts, set variation price instead
				if ( empty( $new_item['item_value'] ) ) {
					$new_item['item_value'] = $product_variation->get_price();
				}
				
				$new_item['item_weight'] = $product_variation->get_weight();

				$product_attribute = wc_get_product_variation_attributes($item['variation_id']);
				$new_item['item_description'] .= ' : ' . current( $product_attribute );

			} else {
				// place 'sku' in a variable before validating using 'empty' to be compatible with PHP v5.4
				$product_sku = $product->get_sku();
				// Ensure id is string and not int
				$new_item['product_id'] = intval( $item['product_id'] );
				$new_item['sku'] = empty( $product_sku ) ? strval( $item['product_id'] ) : $product_sku;

				// If value is empty due to discounts, set product price instead
				if ( empty( $new_item['item_value'] ) ) {
					$new_item['item_value'] = $product->get_price();
				}

				$new_item['item_weight'] = $product->get_weight();
			}

			$new_item += $this->get_label_item_args( $item['product_id'], $args );
			// if( ! empty( $product->post->post_excerpt ) ) {
			// 	$new_item['item_description'] = $product->post->post_excerpt;
			// } elseif ( ! empty( $product->post->post_content ) ) {
			// 	$new_item['item_description'] = $product->post->post_content;
			// }

			array_push($args['items'], $new_item);
		}

		return $args;
	}

	abstract protected function get_label_args_settings( $order_id, $shipany_label_items );

	protected function delete_label_args( $order_id ) {
		return $this->get_shipany_label_tracking( $order_id );
	}

	// Pass args by reference to modify DG if needed
	protected function get_label_item_args( $product_id, &$args ) {
		$new_item = array();
		return $new_item;
	}

	protected function is_cod_payment_method( $order_id ) {
		$is_code = false;
		$order = wc_get_order( $order_id );
		// WC 3.0 comaptibilty
		if ( defined( 'WOOCOMMERCE_VERSION' ) && version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) ) {
			$payment_method = $order->get_payment_method();
			if ( $payment_method == 'cod' ) {
				$is_code = true;
			}
		}
		else {
			if ( isset( $order->payment_method ) && ( $order->payment_method == 'cod' ) ) {
				$is_code = true;
			}
		}

		return $is_code;
	}

	protected function get_wc_currency( $order_id ) {
		$order = wc_get_order( $order_id );
		// WC 3.0 comaptibilty
		if ( defined( 'WOOCOMMERCE_VERSION' ) && version_compare( WOOCOMMERCE_VERSION, '3.0', '>=' ) ) {
			$order_currency = $order->get_currency();
		}
		else {
			$order_currency = $order->get_order_currency();
		}
		return $order_currency;
	}

	/**
	 * Prevents data being copied to subscription renewals
	 */
	public function woocommerce_subscriptions_renewal_order_meta_query( $order_meta_query ) {
		$order_meta_query .= " AND `meta_key` NOT IN ( '_pr_shipment_shipany_label_tracking' )";

		return $order_meta_query;
	}

	/**
	 * Display messages on order view screen
	 */	
	public function render_messages( $current_screen = null ) {
		if ( ! $current_screen instanceof WP_Screen ) {
			$current_screen = get_current_screen();
		}

		if ( isset( $current_screen->id ) && in_array( $current_screen->id, array( 'shop_order', 'edit-shop_order' ), true ) ) {

			$bulk_action_message_opt = get_option( '_shipany_bulk_action_confirmation' );

			if ( ( $bulk_action_message_opt ) && is_array( $bulk_action_message_opt ) ) {

				// $user_id = key( $bulk_action_message_opt );
				// remove first element from array and verify if it is the user id
				$user_id = array_shift( $bulk_action_message_opt );
				if ( get_current_user_id() !== (int) $user_id ) {
					return;
				}

				foreach ($bulk_action_message_opt as $key => $value) {
					$message = wp_kses_post( $value['message'] );
					$type = wp_kses_post( $value['type'] );

					switch ($type) {
                        case 'error':
                            echo '<div class="notice notice-error"><ul><li>' . esc_html($message) . '</li></ul></div>';
                            break;
                        case 'success':
                            echo '<div class="notice notice-success"><ul><li><strong>' . esc_html($message) . '</strong></li></ul></div>';
                            break;
                        default:
                            echo '<div class="notice notice-warning"><ul><li><strong>' . esc_html($message) . '</strong></li></ul></div>';
                    }
				}

				delete_option( '_shipany_bulk_action_confirmation' );
			}
		}
	}
	
	public function validate_bulk_actions( $action, $order_ids ) {
		return '';
	}
	
	public function process_bulk_actions( $action, $order_ids, $orders_count, $shipany_force_product = false, $is_force_product_dom = false ) {
		$label_count = 0;
		$merge_files = array();
		$array_messages = array();

		if ( 'shipany_create_labels' === $action ) {
			
			foreach ( $order_ids as $order_id ) {
				$order = wc_get_order( $order_id );

				try {
					// Create label if one has not been created before
					if( empty( $label_tracking_info = $this->get_shipany_label_tracking( $order_id ) ) ) {

							$this->save_default_shipany_label_items( $order_id );

							// Gather args for API call
							$args = $this->get_label_args( $order_id );

							// Force the use of this Product for all bulk label creation
							if ( $shipany_force_product ) {

								// If forced product is domestic AND order is domestic
								if( $is_force_product_dom && $this->is_shipping_domestic( $order_id ) ) {
									$args['order_details']['shipany_product'] = $shipany_force_product;
								}

								// If forced product is international AND order is international
								if( ! $is_force_product_dom && ! $this->is_shipping_domestic( $order_id ) ) {
									$args['order_details']['shipany_product'] = $shipany_force_product;
								}
							}

							// Allow settings to override saved order data, ONLY for bulk action
							$args = $this->get_bulk_settings_override( $args );
							
							// Allow third parties to modify the args to the APIs
							$args = apply_filters('shipping_shipany_label_args', $args, $order_id );

							$shipany_obj = SHIPANY()->get_shipany_factory();
							$label_tracking_info = $shipany_obj->get_shipany_label( $args );

							$this->save_shipany_label_tracking( $order_id, $label_tracking_info );
							$tracking_note = $this->get_tracking_note( $order_id );
							
							$tracking_note_type = $this->get_tracking_note_type();
							$tracking_note_type = empty( $tracking_note_type ) ? 0 : 1;
							// $label_url = $label_tracking_info['label_url'];

							$order->add_order_note( $tracking_note, $tracking_note_type, true );
							
							++$label_count;

							array_push($array_messages, array(
                                'message' => sprintf( __( 'Order #%s: label Created', 'smart-send-shipping'), $order->get_order_number() ),
                                'type' => 'success',
                            ));

                            do_action( 'pr_shipping_shipany_label_created', $order_id );

					}

					if( ! empty( $label_tracking_info['label_path'] ) ) {
						array_push($merge_files, $label_tracking_info['label_path']);
					}

				} catch (Exception $e) {
					array_push($array_messages, array(
	                    'message' => sprintf( __( 'Order #%s: %s', 'smart-send-shipping'), $order->get_order_number(), $e->getMessage() ),
	                    'type' => 'error',
	                ));
				}
			}

			try {
				
				$file_bulk = $this->merge_label_files( $merge_files );
				
				if ( file_exists( $file_bulk['file_bulk_path'] ) ) {
					// $message .= sprintf( __( ' - %sdownload labels file%s', 'pr-shipping-shipany' ), '<a href="' . $file_bulk['file_bulk_url'] . '" target="_blank">', '</a>' );

	                // We're saving the bulk file path temporarily and access it later during the download process.
		    		// This information expires in 3 minutes (180 seconds), just enough for the user to see the 
		    		// displayed link and click it if he or she wishes to download the bulk labels
					set_transient( '_shipany_bulk_download_labels_file_' . get_current_user_id(), $file_bulk['file_bulk_path'], 180);	

					// Construct URL pointing to the download label endpoint (with bulk param):
					$bulk_download_label_url = $this->generate_download_url( '/' . self::SHIPANY_DOWNLOAD_ENDPOINT . '/bulk' );

					array_push($array_messages, array(
	                    'message' => sprintf( __( 'Bulk labels file created - %sdownload file%s', 'pr-shipping-shipany' ), '<a href="' . $bulk_download_label_url . '" download>', '</a>' ),
	                    'type' => 'success',
	                ));

		        } else {
					array_push($array_messages, array(
	                    'message' => __( 'Could not create bulk  label file, download individually.', 'pr-shipping-shipany' ),
	                    'type' => 'error',
	                ));
		        }

			} catch (Exception $e) {
				array_push($array_messages, array(
                    'message' => $e->getMessage(),
                    'type' => 'error',
                ));
			}
		}

		return $array_messages;
	}

	/**
	 * Generates the download label URL
	 *
	 * @param string $endpoint_path
	 * @return string - The download URL for the label
	 */
	public function generate_download_url( $endpoint_path ) {

		// If we get a different URL addresses from the General settings then we're going to
		// construct the expected endpoint url for the download label feature manually
		if ( site_url() != home_url() ) {

			// You can use home_url() here as well, it really doesn't matter
			// as we're only after for the "scheme" and "host" info.
			$result = parse_url( site_url() );	

			if ( !empty( $result['scheme'] ) && !empty( $result['host'] ) ) {
				return $result['scheme'] . '://' . $result['host'] . $endpoint_path;
			}
		}

		// Defaults to the "Site Address URL" from the general settings along
		// with the the custom endpoint path (with parameters)
		return home_url( $endpoint_path );
	}

	protected function get_bulk_settings_override( $args ) {
		return $args;
	}

	protected function merge_label_files( $files ) {

		if( empty( $files ) ) {
			throw new Exception( __('There are no files to merge.', 'pr-shipping-shipany') );
		}

		if( ! empty( $files[0] ) ) {
			$base_ext = pathinfo($files[0], PATHINFO_EXTENSION);
		} else {
			throw new Exception( __('The first file is empty.', 'pr-shipping-shipany') );
		}

		if ( method_exists( $this, 'merge_label_files_' . $base_ext ) ) {
			return call_user_func( array( $this, 'merge_label_files_' . $base_ext ), $files );
		} else {
			throw new Exception( __('File format not supported.', 'pr-shipping-shipany') );
		}
	}

	/**
	 * Creates a custom endpoint to download the label
	 */
	public function add_download_label_endpoint() {
		add_rewrite_endpoint(  self::SHIPANY_DOWNLOAD_ENDPOINT, EP_ROOT );
		flush_rewrite_rules();
		update_option('shipany_permalinks_flushed', 1);
		//Flush permalink if it is not flushed yet.
		if( !get_option( 'shipany_permalinks_flushed') ){
			flush_rewrite_rules();
			update_option('shipany_permalinks_flushed', 1);
		}
	}

	/**
	 * Processes the download label request
	 *
	 * @return void
	 */
	public function process_download_label() {
	    global $wp_query;

	    if ( ! current_user_can( 'edit_shop_orders' ) ) {
  			return;
  		}
  		
		if ( ! isset($wp_query->query_vars[ self::SHIPANY_DOWNLOAD_ENDPOINT ] ) ) {
			return;
		}
		
	    // If we fail to add the "SHIPANY_DOWNLOAD_ENDPOINT" then we bail, otherwise, we
	    // will continue with the process below.
	    $endpoint_param = $wp_query->query_vars[ self::SHIPANY_DOWNLOAD_ENDPOINT ];
	    if ( ! isset( $endpoint_param ) ) {
	    	return;
	    }

	    $array_messages = get_option( '_shipany_bulk_action_confirmation' );
    	if ( empty( $array_messages ) || !is_array( $array_messages ) ) {
    		$array_messages = array( 'msg_user_id' => get_current_user_id() );
		}

	    if ( $endpoint_param == 'bulk' ) {

	    	$bulk_file_path = get_transient( '_shipany_bulk_download_labels_file_' . get_current_user_id() );

	    	if ( false == $this->download_label( $bulk_file_path ) ) {
	    		array_push($array_messages, array(
                    'message' => __( 'There are currently no bulk label file to download or the download link for the bulk label file has already expired. Please try again.', 'pr-shipping-shipany' ),
                    'type' => 'error'
                ));
			}

			$redirect_url  = admin_url( 'edit.php?post_type=shop_order' );
	    } else {
	    	$order_id = $endpoint_param;

	    	// Get tracking info if it exists
			$label_tracking_info = $this->get_shipany_label_tracking( $order_id );
			// Check whether the label has already been created or not
			if( empty( $label_tracking_info ) ) {
				return;
			}
			
			$label_path = $label_tracking_info['label_path'];

			if ( false == $this->download_label( $label_path ) ) {
	    		array_push($array_messages, array(
                    'message' => __( 'Unable to download file. Label appears to be invalid or is missing. Please try again.', 'pr-shipping-shipany' ),
                    'type' => 'error'
                ));
			}
			
			$redirect_url  = admin_url( 'post.php?post=' . $order_id . '&action=edit' );
	    }

	    update_option( '_shipany_bulk_action_confirmation', $array_messages );

	    // If there are errors redirect to the shop_orders and display error
	    if ( $this->has_error_message( $array_messages ) ) {
            wp_redirect( remove_query_arg( array( '_wp_http_referer', '_wpnonce' ), $redirect_url ) );
            exit;
		}
	}

	/**
	 * Checks whether the current "messages" collection has an
	 * error message waiting to be rendered.
	 *
	 * @param array $messages
	 * @return boolean
	 */
	protected function has_error_message( $messages ) {
		$has_error = false;

		foreach ( $messages as $key => $value ) {
			if ( $value['type'] == 'error' ) {
				$has_error = true;
				break;
			}
		}

		return $has_error;
	}

	/**
	 * Downloads the generated label file
	 *
	 * @param string $file_path
	 * @return boolean|void
	 */
	protected function download_label( $file_path ) {
		if ( !empty( $file_path ) && is_string( $file_path ) && file_exists( $file_path ) ) {
			// Check if buffer exists, then flush any buffered output to prevent it from being included in the file's content
			if ( ob_get_contents() ) {
				ob_clean();
			}

			$filename = basename( $file_path );

		    header( 'Content-Description: File Transfer' );
		    header( 'Content-Type: application/octet-stream' );
		    header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
		    header( 'Expires: 0' );
		    header( 'Cache-Control: must-revalidate' );
		    header( 'Pragma: public' );
		    header( 'Content-Length: ' . filesize( $file_path ) );

		    readfile( $file_path );
		    exit;
		} else {
			return false;
		}
	}

}

endif;
