<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}


/**
 * Shipping Method.
 */

if ( ! class_exists( 'SHIPANY_WC_Method_eCS_Asia' ) ) :

class SHIPANY_WC_Method_eCS_Asia extends WC_Shipping_Method {

	/**
	 * Init and hook in the integration.
	 */
	public function __construct( $instance_id = 0 ) {
		// add_action( 'init', array( $this, 'init' ), 0 );
		$this->id = 'shipany_ecs_asia';
		$this->instance_id = absint( $instance_id );
		$this->method_title = __( 'ShipAny', 'pr-shipping-shipany' ); #shipany
		$this->init();
	}

	/**
	 * init function.
	 */
	private function init() {

		// add_action( 'wp_ajax_update_default_courier', array( $this, 'update_default_courier' ) );
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();
		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
		// add_action( 'woocommerce_after_settings_shipping', array( $this, 'after_load_shipping_page'));
		wp_enqueue_script( 'wc-shipany-setting-js', SHIPANY_PLUGIN_DIR_URL . '/assets/js/shipany-setting.js', array('jquery'), SHIPANY_VERSION );
		wp_localize_script( 'wc-shipany-setting-js', 'shipany_setting_val', array('ajax_url' => admin_url( 'admin-ajax.php' )) );	//can get the value in js by shippany_setting.val_data1
		// wp_enqueue_script( 'wc-shipany-setting-js', SHIPANY_PLUGIN_DIR_URL . '/assets/js/TestConnection.js', array('jquery'), SHIPANY_VERSION );
		// wp_localize_script( 'wc-shipany-setting-js', 'shipany_label_data', $dump );
        wp_enqueue_script( 'wc-dialog-js', SHIPANY_PLUGIN_DIR_URL . '/assets/js/dialog.js', array('jquery'), SHIPANY_VERSION );
    }


	// public function save_fn1() {
	// 	$a='a';
	// }

	// public function save_fn2() {
	// 	$b='b'; //trigger the validation
	// 	WC_Admin_Settings::add_message('Webhook created successfully.');
	// 	if (empty($_POST["woocommerce_shipany_ecs_asia_shipany_default_product_int"])) {
	// 		wc_add_notice( __( 'Please enter your "Udfyld EAN" number.' ), 'error' );
	// 	}
	// 	if (($_POST["woocommerce_shipany_ecs_asia_shipany_api_key"])) {
	// 		wc_print_notice('ha');
	// 	}

	// }

	// public function save_fn3() {
	// 	$c='c';
	// }
	/**
	 * Get message
	 * @return string Error
	 */
	private function get_message( $message, $type = 'notice notice-error is-dismissible' ) {

		ob_start();
		?>
		<div class="<?php echo esc_attr($type) ?>">
			<p><?php echo esc_attr($message) ?></p>
		</div>
		<?php
		return ob_get_clean();
	}

	/**
	 * Initialize integration settings form fields.
	 *
	 * @return void
	 */
	public function init_form_fields() {
		$log_path = SHIPANY()->get_log_url();

		try {
			
			$shipany_obj = SHIPANY()->get_shipany_factory();
			$select_shipany_product_int = $shipany_obj->get_shipany_products_international();

		} catch (Exception $e) {
			SHIPANY()->log_msg( __('Products not displaying - ', 'pr-shipping-shipany') . $e->getMessage() );
		}

		$weight_units = get_option( 'woocommerce_weight_unit' );

		$this->form_fields = array(
            'shipany_api'           => array(
                'title'           => __( 'API Settings', 'pr-shipping-shipany' ),
                'type'            => 'title',
                'description'     => __( 'Please find the information from ShipAny portal, no account? ', 'pr-shipping-shipany' ),
                'class'			  => 'shipany-register-descr',
            ),
			'shipany_api_key' => array(
				'title'             => __( 'API Token', 'pr-shipping-shipany' ),
				'type'              => 'text',
				'description'       => __( 'The API Token (a 36 digits alphanumerical string made from 5 blocks) is required for authentication and you could find it in "Settings" after login ShipAny portal.', 'pr-shipping-shipany' ),
				'desc_tip'          => true,
				'custom_attributes' => array( 'required' => 'required' ),
				'default'           => '',
			),
			'shipany_test_connection_button' => array(
				'title'             => PR_SHIPANY_BUTTON_TEST_CONNECTION,
				'type'              => 'button',
				'custom_attributes' => array(
					'onclick' => "shipanyTestConnection('woocommerce_shipany_ecs_asia_shipany_test_connection_button');"
				),
				'description'       => __( 'Press the button for testing the connection.', 'shipany-for-woocommerce' ),
				'desc_tip'          => true,
			),
			'shipany_default_courier' => array(
				'title'             => __( 'Default Courier', 'pr-shipping-shipany' ),
				'type'              => 'select',
				'description'       => __( 'Please select default courier (you can see a list of available courier after you fill in the API Token and save).', 'pr-shipping-shipany' ),
				'desc_tip'          => true,
				'options'           => $select_shipany_product_int,
				'class'				=> 'wc-enhanced-select default-courier-selector',
				'custom_attributes' => array( 'required' => 'required' )
			),
			'set_default_create' => array(
				'title'             => __( 'Auto Create ShipAny Order', 'pr-shipping-shipany' ),
				'type'              => 'checkbox',
				'label'             => __( ' ', 'pr-shipping-shipany' ),
				'default'           => 'no',
				'description'       => __( 'Please tick here if you want to create ShipAny Order automatically for a new WooCommerce Order', 'pr-shipping-shipany' ),
				'desc_tip'          => true,
			)
		);
	}

	/**
	 * Generate Button HTML.
	 *
	 * @access public
	 * @param mixed $key
	 * @param mixed $data
	 * @since 1.0.0
	 * @return string
	 */
	public function generate_button_html( $key, $data ) {
		$field    = $this->plugin_id . $this->id . '_' . $key;
		$defaults = array(
			'class'             => 'button-secondary',
			'css'               => '',
			'custom_attributes' => array(),
			'desc_tip'          => false,
			'description'       => '',
			'title'             => '',
		);

		$data = wp_parse_args( $data, $defaults );

		ob_start();
		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="<?php echo esc_attr( $field ); ?>"><?php echo wp_kses_post( $data['title'] ); ?></label>
				<?php echo $this->get_tooltip_html( $data ); ?>
			</th>
			<td class="forminp">
				<fieldset>
					<legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
					<button class="<?php echo esc_attr( $data['class'] ); ?>" type="button" name="<?php echo esc_attr( $field ); ?>" id="<?php echo esc_attr( $field ); ?>" style="<?php echo esc_attr( $data['css'] ); ?>" <?php echo $this->get_custom_attribute_html( $data ); ?>><?php echo wp_kses_post( $data['title'] ); ?></button>
					<?php echo $this->get_description_html( $data ); ?>
				</fieldset>
			</td>
		</tr>
		<?php
		return ob_get_clean();
	}

	/**
	 * Processes and saves options.
	 * If there is an error thrown, will continue to save and validate fields, but will leave the erroring field out.
	 */
	public function process_admin_options() {
		try {

			$shipany_obj = SHIPANY()->get_shipany_factory();
			$shipany_obj->shipany_reset_connection();
            $dflt_cour_uid = get_transient('temp_dflt_cour');
            if(!empty($dflt_cour_uid)){
                $this->update_option('shipany_default_courier', $dflt_cour_uid);
				$this->form_fields["shipany_default_courier"]["options"]=get_transient('temp_dflt_cour_name');
            }

		} catch (Exception $e) {

			echo $this->get_message( __('Could not reset connection: ', 'pr-shipping-shipany') . $e->getMessage() );
			// throw $e;
		}
		$api_tk = get_transient('temp_key');
		if ($api_tk ==''){
			$api_tk = $this->settings["shipany_api_key"];
		}
		$temp_api_endpoint = 'https://api.shipany.io/';
		//dev3
        if (strpos($api_tk, 'SHIPANYDEV') !== false){
            $temp_api_endpoint = 'https://api-dev3.shipany.io/';
            $api_tk = str_replace("SHIPANYDEV", "", $api_tk);
        }
		//demo1
        if (strpos($api_tk, 'SHIPANYDEMO') !== false){
            $temp_api_endpoint = 'https://api-demo1.shipany.io/';
            $api_tk = str_replace("SHIPANYDEMO", "", $api_tk);
        }
		$para = [
			"store" => [
				  "domain" => get_permalink( wc_get_page_id( 'shop' )),
				  "src_platform" => "woocommerce", 
				  "meta" => [
					 "store_id" => get_bloginfo( 'name' )
				  ] 
			   ] 
		 ]; 
	
		$shipany_obj = SHIPANY()->get_shipany_factory();
		$response = $shipany_obj->api_client->post_connect('ecommerce/connect/ ', $para,$api_tk, $temp_api_endpoint);

		return parent::process_admin_options();
	}
}

endif;
