<?php
/**
 * @package ShipAny
 */
/*
Plugin Name: ShipAny
Plugin URI: http://wordpress.org/plugins/shipany
Description: ShipAny one-stop logistics platform interconnects WooCommerce to multiple logistics service providers (including SF Express, Kerry Express, Zeek, SF Cold-Chain, Alfred Locker, Hongkong Post, SF Locker, Convenience Store, etc.) so merchants can enjoy full-set features of logistics automation which disrupt the manual logistics process and bring E-Commerce to new generation.
Version: 1.0.10
Author: ShipAny
Author URI: https://www.shipany.io
License: GPLv2 or later
*/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

add_action( 'activated_plugin', 'shipany_activation_redirect');

function shipany_activation_redirect( $plugin ) {
    if( $plugin == plugin_basename( __FILE__ ) ) {
        if(wp_safe_redirect( admin_url( 'admin.php?page=wc-settings&tab=shipping&section=shipany_ecs_asia' ))){
            exit;
        }
    }
}


if ( ! class_exists( 'SHIPANY_WC' ) ) :

class SHIPANY_WC {
	public static $list;
	private $version = "2.5.11";

	protected static $_instance = null;
	
	public $shipping_shipany_order = null;
	
	// protected $shipping_shipany_product = null;

	protected $logger = null;

	private $payment_gateway_titles = array();

	protected $base_country_code = '';

	// 'LI', 'CH', 'NO'
	protected $eu_iso2 = array( 'AT', 'BE', 'BG', 'HR', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE', 'IT', 'LV', 'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SI', 'SK', 'ES', 'SE');

	protected $us_territories = array( 'US', 'GU', 'AS', 'PR', 'UM', 'VI' );
		
	/**
	* Construct the plugin.
	*/
	public function __construct() {
        add_action( 'init', array( $this, 'load_plugin' ), 0 );

		$upload_dir =  wp_upload_dir();
		if ( $file_handle = @fopen( trailingslashit( $upload_dir['basedir'] . '/woocommerce_shipany_label' ) . '.htaccess', 'w' ) ) {
			fwrite( $file_handle, '' );
			fclose( $file_handle );
		}
    }

	/**
	 *
	 * Ensures only one instance is loaded or can be loaded.
	 * @return self Main instance.
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Define WC Constants.
	 */
	private function define_constants() {
		$upload_dir = wp_upload_dir();

		// Path related defines
		$this->define( 'SHIPANY_PLUGIN_FILE', __FILE__ );
		$this->define( 'SHIPANY_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
		$this->define( 'SHIPANY_PLUGIN_DIR_PATH', untrailingslashit( plugin_dir_path( __FILE__ ) ) );
		$this->define( 'SHIPANY_PLUGIN_DIR_URL', untrailingslashit( plugins_url( '/', __FILE__ ) ) );

		$this->define( 'SHIPANY_VERSION', $this->version );

		$this->define( 'SHIPANY_LOG_DIR', $upload_dir['basedir'] . '/wc-logs/' );

		$this->define( 'SHIPANY_ECS_ASIA_TRACKING_URL', 'https://portal.shipany.io/tracking?id=' );
		$this->define( 'PR_SHIPANY_BUTTON_TEST_CONNECTION', __( 'Test Connection', 'shipany-for-woocommerce' ) );
	}
	
	/**
	 * Include required core files used in admin and on the frontend.
	 */
	public function includes() {
		// Auto loader class
		include_once( 'includes/class-shipany-autoloader.php' );
		// Load abstract classes
		include_once( 'includes/abstract-shipany-wc-order.php' );
	

		// Composer autoloader
		include_once( 'vendor/autoload.php' );
	}

	/**
	* Determine which plugin to load.
	*/
	public function load_plugin() {
		// Checks if WooCommerce is installed.
		if ( class_exists( 'WC_Shipping_Method' ) ) {
			$this->base_country_code = $this->get_base_country();

			$shipany_parcel_countries = array('NL', 'BE', 'LU');

			if (!in_array($this->base_country_code, $shipany_parcel_countries) || apply_filters('shipping_shipany_bypass_load_plugin', false)) {
				$this->define_constants();
				$this->includes();
				$this->init_hooks();
                $this->init_ajax_action();
			}
		} else {
			// Throw an admin error informing the user this plugin needs WooCommerce to function
			add_action( 'admin_notices', array( $this, 'notice_wc_required' ) );
		}

	}

    /**
     * Initialize the plugin.
     */
    public function init() {
        add_action( 'admin_notices', array( $this, 'environment_check' ) );
        // $this->get_shipany_wc_product();
        $this->get_shipany_wc_order();
    }

    public function init_hooks() {
        add_action( 'init', array( $this, 'init' ), 1 );
        add_action( 'init', array( $this, 'load_textdomain' ) );

        add_action( 'admin_enqueue_scripts', array( $this, 'shipany_enqueue_scripts') );

        add_action( 'woocommerce_shipping_init', array( $this, 'includes' ) );
        add_filter( 'woocommerce_shipping_methods', array( $this, 'add_shipping_method' ) );

		add_filter( 'woocommerce_states', array( $this, 'custom_woocommerce_states' ));
		add_action( 'wp_ajax_test_shipany_connection', array( $this, 'test_shipany_connection_callback' ) );
		add_action( 'wp_ajax_update_default_courier', array( $this, 'update_default_courier' ) );
    }

	public function update_default_courier() {
		check_ajax_referer( 'shipany-test-con', 'test_con_nonce' );
		$api_key_temp= $_POST['val'];


		try {

			$shipany_obj = $this->get_shipany_factory();
			// $response = $shipany_obj->api_client->get_test_con('couriers/',$api_key_temp);
			// if ( $response->status != 200 ) {
			// 	return;
			// }
			$connection_msg = __('Connection Successful!', 'shipany-for-woocommerce');
			$this->log_msg( $connection_msg );
			$courier_list = $response->body->data->objects;
			self::$list=$courier_list;
			
			
			wp_send_json( array( 
				'connection_success' 	=> $connection_msg,
				'button_txt'			=> PR_SHIPANY_BUTTON_TEST_CONNECTION,
				'courier_list'			=> $courier_list
				) );
				
		} catch (Exception $e) {
			$this->log_msg($e->getMessage());

			wp_send_json( array( 
				'connection_error' => sprintf( __('Connection Failed: %s Make sure to save the settings before testing the connection. ', 'shipany-for-woocommerce'), $e->getMessage() ),
				'button_txt'			=> PR_SHIPANY_BUTTON_TEST_CONNECTION
				 ) );
		}

		wp_die();
	}


	public function test_shipany_connection_callback() {

		$api_key_temp= $_POST['val'];
        $temp_api_endpoint = 'https://api.shipany.io/';
		//dev3
        if (strpos($api_key_temp, 'SHIPANYDEV') !== false){
            $temp_api_endpoint = 'https://api-dev3.shipany.io/';
            $api_key_temp = str_replace("SHIPANYDEV", "", $api_key_temp);
        }

		//demo1
        if (strpos($api_key_temp, 'SHIPANYDEMO') !== false){
            $temp_api_endpoint = 'https://api-demo1.shipany.io/';
            $api_key_temp = str_replace("SHIPANYDEMO", "", $api_key_temp);
        }

        $response = wp_remote_get($temp_api_endpoint.'couriers/', array(
            'headers' => array(
                'api-tk'=> $api_key_temp
            )
        ));
        $status_code = wp_remote_retrieve_response_code($response);
		wp_send_json( array( 
				'connection_success' 	=> $status_code
				) );
		// check_ajax_referer( 'shipany-test-con', 'test_con_nonce' );
		// $api_key_temp= $_POST['val'];
		// try {

		// 	$shipany_obj = $this->get_shipany_factory();
		// 	$response = $shipany_obj->api_client->get_merchant_info_test_con($api_key_temp);
		// 	if ( $response->status != 200 ) {
		// 		return;
		// 	}

		// 	$connection_msg = __('Connection Successful!', 'shipany-for-woocommerce');
		// 	$this->log_msg( $connection_msg );

		// 	wp_send_json( array( 
		// 		'connection_success' 	=> $connection_msg,
		// 		'button_txt'			=> PR_SHIPANY_BUTTON_TEST_CONNECTION
		// 		) );

		// } catch (Exception $e) {
		// 	$this->log_msg($e->getMessage());

		// 	wp_send_json( array( 
		// 		'connection_error' => sprintf( __('Connection Failed: %s Make sure to save the settings before testing the connection. ', 'shipany-for-woocommerce'), $e->getMessage() ),
		// 		'button_txt'			=> PR_SHIPANY_BUTTON_TEST_CONNECTION
		// 		 ) );
		// }

		// wp_die();
	}

	public function custom_woocommerce_states( $states ) {
		$response = wp_remote_get('https://apps.shipany.io/system/sys_param-prod1.json', array( 'sslverify' => false ));
		$json = wp_remote_retrieve_body( $response );
		$jsonArray = json_decode($json,true);
		$hk_distr = $jsonArray["ShipAnyAddrHkDistr2"];

		$hk_distr_formatted = array();
		for ($i = 0; $i < sizeof($hk_distr); $i++){
			$id = $hk_distr[$i]['id'];
			$hk_distr_formatted[$id] = $hk_distr[$i]['name'];
		};
		// $states['HK'] = array(
		//   'HK33' => 'State 1', 
		//   'HK44' => 'State 2'
		// );
		$states['HK'] = $hk_distr_formatted;
		return $states;
	}

	public function get_shipany_wc_order() {
		if ( ! isset( $this->shipping_shipany_order ) ){
			try {
				$shipany_obj = $this->get_shipany_factory();
				
				if( $shipany_obj->is_shipany_ecs_asia() ) {
					$this->shipping_shipany_order = new SHIPANY_WC_Order_eCS_Asia();
				}
				// Ensure folder exists
				$this->shipany_label_folder_check();
			} catch (Exception $e) {
				add_action( 'admin_notices', array( $this, 'environment_check' ) );
			}
		}

		return $this->shipping_shipany_order;
	}

	// public function get_shipany_wc_product() {
	// 	if ( ! isset( $this->shipping_shipany_product ) ){
	// 		try {
	// 			$shipany_obj = $this->get_shipany_factory();
				
	// 			if( $shipany_obj->is_shipany_ecs_asia() ) {
	// 				$this->shipping_shipany_product = new SHIPANY_WC_Product_eCS_Asia();
	// 			}
				
	// 		} catch (Exception $e) {
	// 			add_action( 'admin_notices', array( $this, 'environment_check' ) );
	// 		}
	// 	}

	// 	return $this->shipping_shipany_product;
	// }

	/**
	 * Localisation
	 */
	public function load_textdomain() {
		load_plugin_textdomain( 'pr-shipping-shipany', false, dirname( plugin_basename(__FILE__) ) . '/lang/' );
	}

	public function shipany_enqueue_scripts() {
		// Enqueue Styles
		wp_enqueue_style( 'wc-shipment-shipany-label-css', SHIPANY_PLUGIN_DIR_URL . '/assets/css/shipany-admin.css' );
        wp_enqueue_style( 'wc-shipment-shipany-loader-css', SHIPANY_PLUGIN_DIR_URL . '/assets/css/shipany-loader.css' );
        wp_enqueue_style( 'wc-shipment-shipany-common-css', SHIPANY_PLUGIN_DIR_URL . '/assets/css/shipany-common.css' );

        // Enqueue Scripts
        $screen    = get_current_screen();
        $screen_id = $screen ? $screen->id : '';
		$test_con_data = array(
			'ajax_url'       => admin_url( 'admin-ajax.php' ),
			'loader_image'   => admin_url( 'images/loading.gif' ),
			'test_con_nonce' => wp_create_nonce( 'shipany-test-con' ),
		);
		wp_enqueue_script(
			'wc-shipment-shipany-testcon-js',
			SHIPANY_PLUGIN_DIR_URL . '/assets/js/shipany-test-connection.js',
			array( 'jquery' ),
			SHIPANY_VERSION
		);
		wp_localize_script( 'wc-shipment-shipany-testcon-js', 'shipany_test_con_obj', $test_con_data );
	}


	/**
	 * Define constant if not already set.
	 *
	 * @param  string $name
	 * @param  string|bool $value
	 */
	public function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
	 * Add a new integration to WooCommerce.
	 */
	public function add_shipping_method( $shipping_method ) {
		// Check country somehow
		try {
			$shipany_obj = $this->get_shipany_factory();

			if($shipany_obj->is_shipany_ecs_asia() ) {
				$shipany_ship_meth = 'SHIPANY_WC_Method_eCS_Asia';
				$shipping_method['shipany_ecs'] = $shipany_ship_meth;
			}

		} catch (Exception $e) {
			// do nothing
		}

		return $shipping_method;
	}

	/**
	 * Admin error notifying user that WC is required
	 */
	public function notice_wc_required() {
	?>
		<div class="error">
			<p><?php _e( 'requires WooCommerce to be installed and activated!', 'pr-shipping-shipany' ); ?></p>
		</div>
	<?php
	}

	/**
	 * environment_check function.
	 */
	public function environment_check() {
		// Try to get the shipany object...if exception if thrown display to user, mainly to check country support.
		try {
			$this->get_shipany_factory();
		} catch (Exception $e) {
			echo '<div class="error"><p>' . esc_html($e->getMessage()) . '</p></div>';
		}
	}

	public function get_base_country() {
		$country_code = wc_get_base_location();
		return apply_filters( 'shipping_shipany_base_country', $country_code['country'] );
	}

	/**
	 * Create an object from the factory based on country.
	 */
	public function get_shipany_factory() {

		$base_country_code = $this->get_base_country();
		
		try {	
			$shipany_obj = SHIPANY_API_Factory::make_shipany( $base_country_code );		
		} catch (Exception $e) {
			throw $e;
		}

		return $shipany_obj;
	}

	public function get_shipany_factory_test_con($api_key_temp) {

		$base_country_code = $this->get_base_country();
		
		try {	
			$shipany_obj = SHIPANY_API_Factory::make_shipany_test_con( $base_country_code,$api_key_temp );		
		} catch (Exception $e) {
			throw $e;
		}

		return $shipany_obj;
	}

	public function get_api_url() {

		try {

			$shipany_obj = $this->get_shipany_factory();
			
			if( $shipany_obj->is_shipany_ecs_asia() ) {

				return $shipany_obj->get_api_url();

			}
			
		} catch (Exception $e) {
			throw new Exception('Cannot get shipany api credentials!');			
		}
	}

	public function get_shipping_shipany_settings( ) {
		$shipany_settings = array();

		try {
			$shipany_obj = $this->get_shipany_factory();
			
			if( $shipany_obj->is_shipany_ecs_asia() ) {
				$shipany_settings = $shipany_obj->get_settings();
			}

		} catch (Exception $e) {
			throw $e;
		}

		return $shipany_settings;
	}

	public function log_msg( $msg )	{

		try {
			$shipping_shipnay_settings = $this->get_shipping_shipany_settings();
			$shipany_debug = isset( $shipping_shipnay_settings['shipany_debug'] ) ? $shipping_shipnay_settings['shipany_debug'] : 'yes';
			
			if( ! $this->logger ) {
				$this->logger = new SHIPANY_Logger( $shipany_debug );
			}

			$this->logger->write( $msg );
			
		} catch (Exception $e) {
			// do nothing
		}
	}

	public function get_log_url( )	{

		try {
			$shipping_shipnay_settings = $this->get_shipping_shipany_settings();
			$shipany_debug = isset( $shipping_shipnay_settings['shipany_debug'] ) ? $shipping_shipnay_settings['shipany_debug'] : 'yes';
			
			if( ! $this->logger ) {
				$this->logger = new SHIPANY_Logger( $shipany_debug );
			}
			
			return $this->logger->get_log_url( );
			
		} catch (Exception $e) {
			throw $e;
		}
	}

	/**
	 * Function return whether the sender and receiver country is the same territory
	 */
	public function is_shipping_domestic( $country_receiver ) {   	 

		// If base is US territory
		if( in_array( $this->base_country_code, $this->us_territories ) ) {
			
			// ...and destination is US territory, then it is "domestic"
			if( in_array( $country_receiver, $this->us_territories ) ) {
				return true;
			} else {
				return false;
			}

		} elseif( $country_receiver == $this->base_country_code ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Function return whether the sender and receiver country is "crossborder" i.e. needs CUSTOMS declarations (outside EU)
	 */
	public function is_crossborder_shipment( $country_receiver ) {

		if ($this->is_shipping_domestic( $country_receiver )) {
			return false;
		}

		// Is sender country in EU...
		if ( in_array( $this->base_country_code, $this->eu_iso2 ) ) {
			// ... and receiver country is in EU means NOT crossborder!
			if ( in_array( $country_receiver, $this->eu_iso2 ) ) {
				return false;
			} else {
				return true;
			}
		} else {
			return true;
		}
	}

	/**
     * Installation functions
     *
     * Create temporary folder and files. labels will be stored here as required
     *
     * empty_pdf_task will delete them hourly
     */
    public function create_shipany_label_folder() {
        // Install files and folders for uploading files and prevent hotlinking
        $upload_dir =  wp_upload_dir();

        $files = array(
            array(
                'base'      => $upload_dir['basedir'] . '/woocommerce_shipany_label',
                'file'      => '.htaccess',
                'content'   => ''
            ),
            array(
                'base'      => $upload_dir['basedir'] . '/woocommerce_shipany_label',
                'file'      => 'index.html',
                'content'   => ''
            )
        );

        foreach ( $files as $file ) {

            if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {

                if ( $file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ) ) {
                    fwrite( $file_handle, $file['content'] );
                    fclose( $file_handle );
                }

            }

        }
    }

    public function shipany_label_folder_check() {
        $upload_dir =  wp_upload_dir();
            $this->create_shipany_label_folder();
    }

    public function get_shipany_label_folder_dir() {
        $upload_dir =  wp_upload_dir();
            return $upload_dir['basedir'] . '/woocommerce_shipany_label/';
        return '';
    }

    public function get_shipany_label_folder_url() {
        $upload_dir =  wp_upload_dir();
            return $upload_dir['baseurl'] . '/woocommerce_shipany_label/';
        return '';
    }
    public function init_ajax_action(){
        // hook into admin-ajax
        // the text after 'wp_ajax_' and 'wp_ajax_no_priv_' in the add_action() calls
        // that follow is what you will use as the value of data.action in the ajax
        // call in your JS
        // if the ajax call will be made from JS executed when user is logged into WP,
        // then use this version
        add_action ('wp_ajax_on_change_load_couriers', array($this, 'on_change_load_couriers')) ;
        // if the ajax call will be made from JS executed when no user is logged into WP,
        // then use this version
        add_action ('wp_ajax_nopriv_on_change_load_couriers', array($this, 'on_change_load_couriers'));
        add_action('wp_ajax_set_default_courier', array($this, 'set_default_courier'));
        add_action ('wp_ajax_nopriv_on_change_set_default_courier', array($this, 'set_default_courier')) ;
    }

    public function on_change_load_couriers(){
        if (!isset ($_POST['api_tk'])) {
            // set the return value you want on error
            // return value can be ANY data type (e.g., array())
            $return_value = 'Invalid API token' ;

            wp_send_json_error ($return_value) ;
        }

        $api_tk = $_POST['api_tk'];
        $temp_api_endpoint = 'https://api.shipany.io/';

		// dev3
        if (strpos($api_tk, 'SHIPANYDEV') !== false){
            $temp_api_endpoint = 'https://api-dev3.shipany.io/';
            $api_tk = str_replace("SHIPANYDEV", "", $api_tk);
        }

		//demo1
        if (strpos($api_tk, 'SHIPANYDEMO') !== false){
            $temp_api_endpoint = 'https://api-demo1.shipany.io/';
            $api_tk = str_replace("SHIPANYDEMO", "", $api_tk);
        }

        $response = wp_remote_get($temp_api_endpoint.'couriers/', array(
            'headers' => array(
                'api-tk'=> $api_tk
            )
        ));
        $status_code = wp_remote_retrieve_response_code($response);
        $body = wp_remote_retrieve_body($response);
        if ($status_code !== 200){
            $resp_body = json_decode($body);
            $error_title = $resp_body->result->descr;
            $error_detail = implode('.', $resp_body->result->details);
            $return_value = array(
                'success' => false,
                'data' => array(
                    'error_title' => $error_title,
                    'error_detail' => $error_detail
                )
            );

            wp_send_json_error ($return_value);
        }

        $body = json_decode($body)->data->objects;
        // do processing you want based on $id
        $rv_cour_list = array();
        foreach ($body as $key => $value){
            $rv_cour_list[$value->uid] = $value->name;
        }
        // set the return value you want on success
        // return value can be ANY data type (e.g., array())
        $return_value = array( 'success' => true, 'data' => $rv_cour_list);

        wp_send_json_success ($return_value) ;
    }

    public function set_default_courier(){
        $default_courier = $_POST['cour_uid'];
		$default_courier_name = $_POST['cour_name'];
		$temp_key = $_POST['temp_key'];
        $is_success = set_transient('temp_dflt_cour', $default_courier, 600);
		$is_success_name = set_transient('temp_dflt_cour_name', $default_courier_name, 600);
		$is_success_name = set_transient('temp_key', $temp_key, 600);
        $return_value = array( 'success' => $is_success);
        wp_send_json_success ($return_value) ;
    }

}

endif;

if( ! function_exists('SHIPANY') ) {

	/**
	 * Activation hook.
	 */
	function shipany_activate() {
		// Flag for permalink flushed
		update_option('shipany_permalinks_flushed', 0);
	}
	register_activation_hook( __FILE__, 'shipany_activate' );

	function SHIPANY() {
		return SHIPANY_WC::instance();
	}

	$SHIPANY_WC = SHIPANY();
}
