jQuery(function ($) {
    const commonUtils = {
        isValidUUID: function (paramString) {
            const uuidV4Regex = /^[A-F\d]{8}-[A-F\d]{4}-4[A-F\d]{3}-[89AB][A-F\d]{3}-[A-F\d]{12}$/i;
            return uuidV4Regex.test(paramString.replace('SHIPANYDEV','').replace('SHIPANYDEMO',''));
        },
        getOffset: function (elem){
            const rect = elem.getBoundingClientRect();
            return {
                left: rect.left + window.scrollX,
                top: rect.top + window.scrollY,
                width: rect.width,
                height: rect.height
            };
        }
    }
    const appendLoader = function (cssSelector){
        $(cssSelector).after('<div class="lds-dual-ring"></div>');
        let targetElementInDom = $.find(cssSelector)
        let loaderDom = $.find('.lds-dual-ring');
        if(Array.isArray(targetElementInDom) && targetElementInDom.length > 0){
            targetElementInDom = targetElementInDom[0]
            const position = commonUtils.getOffset(targetElementInDom)
            let loaderPosition = position.left + position.width - 176;
            loaderDom[0].style.left = `${loaderPosition}px`;
        }
    }
    const removeLoader = function (){
        let loaderDoms = $.find('.lds-dual-ring');
        if(Array.isArray(loaderDoms) && loaderDoms.length > 0){
            for(const elem of loaderDoms){
                elem.remove();
            }
        }
    }

    const appendRegisterLink = function (){
        let textElem = $.find('.shipany-register-descr + p');
        if(Array.isArray(textElem) && textElem.length > 0){
            textElem = textElem[0];
            $('.shipany-register-descr + p').append('<a target="_blank" href="https://portal.shipany.io/user/register?referrer=woocommerce">Register now</a>')
            $('.shipany-register-descr').hide();
        }
    }

    var wc_shipping_setting = {
        // init Class
        init: function () {
            appendRegisterLink();
            let isLoadedCourierList = false;
            let isLoading = false;
            let currentInputToken = $("input[name='woocommerce_shipany_ecs_asia_shipany_api_key']").val();
            $("input[name='woocommerce_shipany_ecs_asia_shipany_api_key']").bind('onChangeAccessToken', function (e) {
                // $("button[name='save']").trigger('click');
                let targetTokenVal = e.target.value;
                // if(currentInputToken === targetTokenVal){
                //     return;
                // }else{
                //     isLoadedCourierList = false;
                // }
                isLoadedCourierList = false;
                if (commonUtils.isValidUUID(targetTokenVal)) {
                    console.log('is UUID');
                    if (!isLoadedCourierList) {
                        // isLoadedCourierList = true
                        if(isLoading){
                            return;
                        }
                        console.log('Going to trigger API call');
                        $('.default-courier-selector').prop('disabled', true);
                        appendLoader("input[name='woocommerce_shipany_ecs_asia_shipany_api_key");
                        isLoading = true;
                        $.ajax({
                            url: shipany_setting_val.ajax_url,
                            method: 'POST',
                            dataType: 'JSON',
                            data:{
                                // the value of data.action is the part AFTER 'wp_ajax_' in
                                // the add_action ('wp_ajax_xxx', 'yyy') in the PHP above
                                action: 'on_change_load_couriers',
                                // ANY other properties of data are passed to your_function()
                                // in the PHP global $_REQUEST (or $_POST in this case)
                                api_tk : targetTokenVal
                            },
                            success: function (response){
                                isLoading = false
                                if(response.success){
                                    console.log('get response')
                                    const courierList = {...response.data.data}
                                    var optionsAsString = "";

                                    // check if need remove old mounted dom here
                                    $('.default-courier-selector option').each(function() {
                                        $(this).remove();
                                    });
                                    for(const courierUUID of Object.keys(courierList)){
                                        optionsAsString += "<option value='" + courierUUID + "'>" + courierList[courierUUID] + "</option>";
                                    }
                                    $( 'select[name="woocommerce_shipany_ecs_asia_shipany_default_courier"]' ).append( optionsAsString );

                                    $(".default-courier-selector option[value='']").each(function() {
                                        $(this).remove();
                                    });
                                }else{
                                    console.log('failed to get success')
                                    $('.default-courier-selector option').each(function() {
                                        $(this).remove();
                                    });
                                    let errorTitle = response.data.data.error_title;
                                    let errorDetail = response.data.data.error_detail;
                                    $('.shipany-main-dialog').show();
                                    $('.shipany-dialog > .title').text(errorTitle);
                                    $('.shipany-dialog > .detail').text(errorDetail);
                                    isLoadedCourierList = false;
                                    isLoading = false;
                                }
                                isLoadedCourierList = true;
                                currentInputToken = targetTokenVal;
                                removeLoader();
                                $('.default-courier-selector').prop('disabled', false);
                            },
                            error: function (xhr, ajaxOptions, thrownError){
                                console.log('error');
                                isLoadedCourierList = false;
                                isLoading = false;

                            }
                        })
                    }
                } else {
                    console.log('No an UUID, do nothing');
                    $('.default-courier-selector option').each(function() {
                        $(this).remove();
                    });
                }        
            }).change(function () {
                $(this).trigger('onChangeAccessToken'); //call onChangeAccessToken on blur
            }).keyup(function (e) {
                var code = (e.keyCode || e.which);
                if(code == 17) {
                    return;
                }
                $(this).trigger('onChangeAccessToken'); //call onChangeAccessToken on blur
            });

            // handle in case end user select default courier from rendered options
            $('.default-courier-selector').bind('onChangeSelectDefaultCourier', function (e){
                const targetValue = e.target.value;
                const targetText = $(".default-courier-selector option:selected").text();
                const tempKey= document.getElementById('woocommerce_shipany_ecs_asia_shipany_api_key').value;
                console.log(targetValue);
                console.log(targetText);
                $.ajax({
                    url: shipany_setting_val.ajax_url,
                    method: 'POST',
                    dataType: 'JSON',
                    data: {
                        action: 'set_default_courier',
                        cour_uid: targetValue,
                        cour_name: targetText,
                        temp_key: tempKey
                    },
                    success: function (response){
                        if(response.success){
                            console.log('get response');
                        }
                    },
                    error: function (xhr, ajaxOptions, thrownError){
                        console.log('error')
                    }

                })
            }).change(function (){
                $(this).trigger('onChangeSelectDefaultCourier'); //call onChangeAccessToken on blur
            })
        },
    };

    wc_shipping_setting.init();
    console.log('test');

});
